import logging
log = logging.getLogger(__name__)

import numpy
from pubsub import pub
import wx

from ....config.measurement import MeasurementConfigPanel
from ....tool.box import Dialog, MessageDialog

try:
	from ..surface import SurfacePlot
except ImportError as e:
	plot_available = False
	log.debug('Could not import SurfacePlot: {0}'.format(str(e)))
else:
	plot_available = True

"""
A historical live view plot for list values.
"""


class PlotSettings(object):
	"""
	Wrapper for all the settings configured via dialog.
	"""

	def __init__(self):
		self.enabled = plot_available
		self.num_lines = 100


class PlotSettingsDialog(Dialog):
	"""
	Set up the live view plot.
	"""

	def __init__(self, parent, ok_callback, *args, **kwargs):
		Dialog.__init__(self, parent=parent, title='Plot settings')

		self.ok_callback = ok_callback

		dialog_box = wx.BoxSizer(wx.VERTICAL)

		# Enabled.
		self.enabled_checkbox = wx.CheckBox(self, label='Enabled')
		if not plot_available:
			self.enabled_checkbox.Disable()
		dialog_box.Add(self.enabled_checkbox, flag=wx.ALL, border=5)

		# Capture.
		capture_static_box = wx.StaticBox(self, label='Capture')
		capture_box = wx.StaticBoxSizer(capture_static_box, wx.VERTICAL)
		capture_sizer = wx.FlexGridSizer(rows=2, cols=2, hgap=5)
		capture_box.Add(capture_sizer, flag=wx.CENTER)
		dialog_box.Add(capture_box, flag=wx.EXPAND|wx.ALL, border=5)

		## Number of lines.
		capture_sizer.Add(wx.StaticText(self, label='Lines:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.lines_input = wx.SpinCtrl(self, min=2, max=1e4, initial=100)
		capture_sizer.Add(self.lines_input, flag=wx.CENTER)

		# End buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER)

		ok_button = wx.Button(self, wx.ID_OK)
		self.Bind(wx.EVT_BUTTON, self.OnOk, ok_button)
		button_box.Add(ok_button)

		cancel_button = wx.Button(self, wx.ID_CANCEL)
		button_box.Add(cancel_button)

		self.SetSizerAndFit(dialog_box)

	def OnOk(self, evt=None):
		self.ok_callback(self)

		self.Destroy()

	def GetValue(self):
		plot_settings = PlotSettings()
		plot_settings.enabled = self.enabled_checkbox.Value
		plot_settings.num_lines = self.lines_input.Value

		return plot_settings

	def SetValue(self, plot_settings):
		self.enabled_checkbox.Value = plot_settings.enabled
		self.lines_input.Value = plot_settings.num_lines


class ListLiveViewPanel(wx.Panel):
	"""
	A panel to display a live view plot of a list resource.
	"""

	def __init__(self, parent, global_store, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.global_store = global_store
		self._measurement_resource_name = None

		# Defaults.
		self.plot_settings = PlotSettings()

		self.enabled = False
		self.capturing_data = False
		self.resource_backup = None

		# Initialize recorded values.
		self.init_values()

		# Plot and toolbar.
		display_box = wx.BoxSizer(wx.VERTICAL)

		## Plot.
		if plot_available:
			self.plot = SurfacePlot(self, style='waveform')
			display_box.Add(self.plot.control, proportion=1, flag=wx.EXPAND)

			self.plot.x_label = 'Waveform time (s)'
			self.plot.y_label = 'History'
		else:
			display_box.Add((500, -1), proportion=1, flag=wx.EXPAND)

		## Controls.
		if plot_available:
			controls_box = wx.BoxSizer(wx.HORIZONTAL)
			display_box.Add(controls_box, flag=wx.CENTER|wx.ALL, border=5)

			### Capture.
			capture_static_box = wx.StaticBox(self, label='Control')
			capture_box = wx.StaticBoxSizer(capture_static_box)
			controls_box.Add(capture_box, flag=wx.CENTER)

			self.reset_button = wx.Button(self, label='Reset')
			self.Bind(wx.EVT_BUTTON, self.OnReset, self.reset_button)
			capture_box.Add(self.reset_button, flag=wx.CENTER)

			### Settings.
			settings_static_box = wx.StaticBox(self, label='Settings')
			settings_box = wx.StaticBoxSizer(settings_static_box, wx.HORIZONTAL)
			controls_box.Add(settings_box, flag=wx.CENTER|wx.LEFT, border=10)

			self.plot_settings_button = wx.Button(self, label='Plot...')
			self.Bind(wx.EVT_BUTTON, self.OnPlotSettings, self.plot_settings_button)
			settings_box.Add(self.plot_settings_button, flag=wx.CENTER)

		self.SetSizer(display_box)

		# Subscriptions.
		pub.subscribe(self.msg_data_capture_start, 'data_capture.start')
		pub.subscribe(self.msg_data_capture_data, 'data_capture.data')
		pub.subscribe(self.msg_data_capture_stop, 'data_capture.stop')

	@property
	def measurement_resource_name(self):
		if self._measurement_resource_name is None:
			return ''
		else:
			return self._measurement_resource_name

	@measurement_resource_name.setter
	def measurement_resource_name(self, value):
		if value:
			self._measurement_resource_name = value
			try:
				self.resource = self.global_store.resources[self._measurement_resource_name]
			except KeyError:
				self.resource = None
		else:
			self._measurement_resource_name = None
			self.resource = None

	def init_values(self):
		"""
		Clear captured values.
		"""

		self._lines = None
		self.time_range = (0.0, 0.0)

	def update_plot(self):
		"""
		Redraw the plot.
		"""

		# Wait for at least one line.
		if self._lines is None:
			self.plot.surface_data = None
		else:
			self.plot.surface_data = (self._lines, self.time_range, (1, len(self._lines)))

		wx.CallAfter(self.plot.redraw)

	def add_values(self, values):
		"""
		Update the plot with a new list of values.
		"""

		if not self.plot_settings.enabled:
			return

		# Extract the times and the data values.
		times, values = zip(*values)
		time_range = min(times), max(times)

		# Sanity check, since the new values must match existing ones.
		if self._lines is not None:
			if len(self._lines[-1]) != len(values):
				log.warning('Data length mismatch: was {0}, became {1}'.format(len(self._lines[-1]), len(values)))
				self.init_values()
			elif self.time_range != time_range:
				log.warning('Time range mismatch: was {0}, became {1}'.format(self.time_range, time_range))
				self.init_values()

		# Update values.
		if self._lines is None:
			self._lines = numpy.array([values])
			self.time_range = time_range
		else:
			self._lines = numpy.append(self._lines, [values], 0)

		cut_idx = len(self._lines) - self.plot_settings.num_lines
		if cut_idx > 0:
			self._lines = self._lines[cut_idx:]

		# Plot.
		self.update_plot()

	def close(self):
		"""
		Perform cleanup.
		"""

		# Unsubscriptions.
		pub.unsubscribe(self.msg_data_capture_start, 'data_capture.start')
		pub.unsubscribe(self.msg_data_capture_data, 'data_capture.data')
		pub.unsubscribe(self.msg_data_capture_stop, 'data_capture.stop')

	def OnReset(self, evt=None):
		self.init_values()
		self.update_plot()

	def OnPlotSettings(self, evt=None):
		"""
		Open the plot settings dialog.
		"""

		def ok_callback(dlg):
			self.plot_settings = dlg.GetValue()

		dlg = PlotSettingsDialog(self, ok_callback)
		dlg.SetValue(self.plot_settings)
		dlg.Show()

	def msg_data_capture_start(self, name):
		if name == self.measurement_resource_name:
			if self.enabled:
				self.capturing_data = True

	def msg_data_capture_data(self, name, value):
		if name == self.measurement_resource_name:
			if self.capturing_data:
				self.add_values(value)

	def msg_data_capture_stop(self, name):
		if name == self.measurement_resource_name:
			if self.capturing_data:
				self.capturing_data = False


class ListMeasurementFrame(wx.Frame):
	def __init__(self, parent, global_store, *args, **kwargs):
		wx.Frame.__init__(self, parent, *args, **kwargs)

		# Frame.
		frame_box = wx.BoxSizer(wx.VERTICAL)

		## Measurement setup.
		self.measurement_config_panel = MeasurementConfigPanel(self, global_store, scaling=False)
		frame_box.Add(self.measurement_config_panel, flag=wx.EXPAND)

		## Live view.
		self.live_view_panel = ListLiveViewPanel(self, global_store)
		frame_box.Add(self.live_view_panel, proportion=1, flag=wx.EXPAND)

		self.SetSizerAndFit(frame_box)

		self.Bind(wx.EVT_CLOSE, self.OnClose)

	def OnClose(self, evt):
		if self.live_view_panel.capturing_data:
			msg = 'Cannot close, as a sweep is currently in progress.'
			MessageDialog(self, msg, 'Sweep in progress').Show()

			evt.Veto()
			return

		self.measurement_config_panel.close()
		self.live_view_panel.close()

		evt.Skip()



from nose.tools import eq_
from numpy import linspace
from threading import Lock
import time
from unittest import main, TestCase

from spacq.tests.tool.box import AssertHandler

from ..units import Quantity

from .. import resources


class WithAttribute(object):
	x = 5


class WithMethods(object):
	_x = 5

	def get_x(self):
		return self._x

	def set_x(self, value):
		self._x = value


class WithProperty(object):
	_x = 5

	@property
	def x(self):
		return self._x

	@x.setter
	def x(self, value):
		self._x = value


class ResourceTest(TestCase):
	def testUseless(self):
		"""
		A resource with neither a getter nor a setter.
		"""

		res = resources.Resource()

		try:
			res.value
		except resources.NotReadable:
			pass
		else:
			assert False, 'Expected NotReadable.'

		assert not res.readable

		try:
			res.value = 5
		except resources.NotWritable:
			pass
		else:
			assert False, 'Expected NotWritable.'

		assert not res.writable

	def testMoreThanUseless(self):
		"""
		A resource with a lookup getter or setter, but no object.
		"""

		# Broken from the start.
		try:
			res = resources.Resource(getter='x')
		except:
			pass
		else:
			assert False, 'Expected ValueError.'

		try:
			res = resources.Resource(setter='x')
		except:
			pass
		else:
			assert False, 'Expected ValueError.'

		# Broken later on.
		res = resources.Resource()
		res.getter = 'x'
		res.setter = 'x'

		try:
			res.value
		except resources.NotReadable:
			pass
		else:
			assert False, 'Expected NotReadable.'

		try:
			res.value = 5
		except resources.NotWritable:
			pass
		else:
			assert False, 'Expected NotWritable.'

	def testReadonly(self):
		"""
		A read-only resource using an attribute.
		"""

		dev = WithAttribute()
		res = resources.Resource(dev, 'x')

		eq_(res.value, 5)

		try:
			res.value = 55
		except resources.NotWritable:
			pass
		else:
			assert False, 'Expected NotWritable.'

		eq_(res.value, 5)

	def testWriteonly(self):
		"""
		A write-only resource using an attribute.
		"""

		dev = WithAttribute()
		res = resources.Resource(dev, None, 'x')

		eq_(dev.x, 5)

		try:
			res.value
		except resources.NotReadable:
			pass
		else:
			assert False, 'Expected NotReadable.'

		res.value = 55

		eq_(dev.x, 55)

	def testWithMethods(self):
		"""
		A read-write resource using methods.
		"""

		dev = WithMethods()
		res = resources.Resource(None, dev.get_x, dev.set_x)

		eq_(res.value, 5)

		res.value = 1234

		eq_(res.value, 1234)

	def testWithProperty(self):
		"""
		A read-write resource using a property.
		"""

		dev = WithProperty()
		res = resources.Resource(dev, 'x', 'x')

		eq_(res.value, 5)

		res.value = 5678

		eq_(res.value, 5678)

	def testConverter(self):
		"""
		Conversion with and without a converter.
		"""

		dev = WithAttribute()
		res1 = resources.Resource(dev)
		res2 = resources.Resource(dev, converter=int)
		res3 = resources.Resource(dev, converter=float)

		eq_(res1.convert('5'), '5')
		eq_(res2.convert('5') / 2, 2)
		eq_(res3.convert('5') / 2, 2.5)

	def testAllowedValues(self):
		"""
		Verify that only allowed values are allowed.
		"""

		allowed = set([5, 10, 15])

		dev = WithAttribute()
		res = resources.Resource(dev, 'x', 'x', allowed_values=allowed)

		for value in allowed:
			res.value = value
			eq_(res.value, value)

		try:
			res.value = -5
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError'

		eq_(res.allowed_values, allowed)

	def testWrapping(self):
		"""
		Wrap resources in other resources, and then undo.
		"""

		dev = WithAttribute()
		res1 = resources.Resource(dev, 'x', 'x')

		res1.value = 5
		eq_(res1.value, 5)

		# Wrap once.
		res2 = res1.wrapped('wrapper1', setter_filter=lambda x: 3 * x)

		eq_(res2.value, 5)
		res2.value = 5
		eq_(res1.value, 15)
		eq_(res2.value, 15)

		# Wrap again.
		res3 = res2.wrapped('wrapper2', lambda x: 5 * x)

		eq_(res3.value, 75)
		res3.value = 10
		eq_(res1.value, 30)
		eq_(res2.value, 30)
		eq_(res3.value, 150)

		# Unwrap.
		res4 = res3.unwrapped('wrapper1')

		eq_(res4.value, 150)
		res4.value = 7
		eq_(res1.value, 7)
		eq_(res2.value, 7)
		eq_(res3.value, 35)
		eq_(res4.value, 35)

		# Modify the original Resource.
		res1.value = 1
		eq_(res1.value, 1)
		eq_(res2.value, 1)
		eq_(res3.value, 5)
		eq_(res4.value, 5)

		assert not res1.is_wrapped_by('wrapper1')
		assert not res1.is_wrapped_by('wrapper2')
		assert     res2.is_wrapped_by('wrapper1')
		assert not res2.is_wrapped_by('wrapper2')
		assert     res3.is_wrapped_by('wrapper1')
		assert     res3.is_wrapped_by('wrapper2')
		assert not res4.is_wrapped_by('wrapper1')
		assert     res4.is_wrapped_by('wrapper2')

	def testSweep(self):
		"""
		Ramp up and down.
		"""

		buf = None
		def setter(value):
			if abs(value) > 10:
				raise ValueError(value)

			buf.append(value)

		exceptions = None
		def exception_callback(e):
			exceptions.append(tuple(e))

		res = resources.Resource(setter=setter)

		# Check the values.
		buf = []
		res.sweep(1.0, 5.0, 5)
		eq_(buf, list(linspace(1.0, 5.0, 5)))

		# Check the time.
		buf = []
		start_time = time.time()
		res.sweep(-5.0, 5.0, 11, delay=0.05)
		time_diff = time.time() - start_time
		assert time_diff > 0.55
		assert time_diff < 0.75
		eq_(buf, list(linspace(-5.0, 5.0, 11)))

		# Check the exceptions.
		buf = []
		exceptions = []
		res.sweep(9.0, 12.0, 4, delay=0.05, exception_callback=exception_callback)
		res.sweep(-15.0, 0.0, 3, delay=0.05, exception_callback=exception_callback)
		eq_(buf, list(linspace(9.0, 10.0, 2)))
		eq_(exceptions, [(11.0,), (-15.0,)])

	def testDimensions(self):
		"""
		Ensure that dimensions for values are verified in both directions.
		"""

		value = []

		res1 = resources.Resource(getter=lambda: value[-1], setter=lambda x: value.append(x))
		res1.units = 'ns.V2.pJ-1'

		# Valid.
		## Without wrapping.
		res1.value = Quantity(12.34, 'kg.m2.s-3.A-2')
		eq_(res1.value, Quantity(12340, 'g.m2.s-3.A-2'))

		## With wrapping.
		res2 = res1.wrapped('w1', getter_filter=lambda x: 2 * x, setter_filter=lambda x: 3 * x)
		res2.value = Quantity(12.34, 'kg.m2.s-3.A-2')
		eq_(res2.value, Quantity(74040, 'g.m2.s-3.A-2'))

		# Invalid.
		## Dimensions.
		try:
			res1.value = Quantity(12.34, 's')
		except TypeError:
			pass
		else:
			assert False, 'Expected TypeError'

		## Not even a quantity.
		try:
			res1.value = 12.34
		except TypeError:
			pass
		else:
			assert False, 'Expected TypeError'

		## Also on read.
		value.append(12.34)

		try:
			res1.value
		except TypeError:
			pass
		else:
			assert False, 'Expected TypeError'

		## And with wrapping.
		res2 = res1.wrapped('w2', getter_filter=lambda x: Quantity(-9.1, 'A'), setter_filter=lambda x: 6.3)

		try:
			res2.value
		except TypeError:
			pass
		else:
			assert False, 'Expected TypeError'

		try:
			res2.value = Quantity(12.34, 'ns.V2.pJ-1')
		except TypeError:
			pass
		else:
			assert False, 'Expected TypeError'


class AcquisitionThreadTest(TestCase):
	def testWithoutResource(self):
		"""
		Let the thread run without a resource.
		"""

		buf = []
		delay = Quantity(30, 'ms')
		
		thr = resources.AcquisitionThread(delay, buf.append)

		thr.start()
		time.sleep(delay.value * 4.5)
		thr.done = True

		eq_(buf, [])

	def testWithResource(self):
		"""
		Let the thread run with a resource.
		"""

		dev = WithMethods()
		res = resources.Resource(dev, dev.get_x)

		expected = [res.value] * 5

		buf = []
		delay = Quantity(30, 'ms')

		thr = resources.AcquisitionThread(delay, buf.append, res)

		thr.start()
		time.sleep(delay.value * 4.5)
		thr.done = True

		eq_(buf, expected)

	def testWithLock(self):
		"""
		Pause the thread with a lock.
		"""

		dev = WithMethods()
		res = resources.Resource(dev, dev.get_x)
		lock = Lock()

		expected = [res.value] * 4

		buf = []
		delay = Quantity(30, 'ms')

		thr = resources.AcquisitionThread(delay, buf.append, res, running_lock=lock)

		thr.start()
		time.sleep(delay.value * 1.5)
		lock.acquire()
		time.sleep(delay.value * 5)
		lock.release()
		time.sleep(delay.value * 1.5)
		thr.done = True

		eq_(buf, expected)

	def testWithUnreadableResource(self):
		"""
		Watch the errors come rolling in.
		"""

		res = resources.Resource()

		buf = []
		delay = Quantity(30, 'ms')

		thr = resources.AcquisitionThread(delay, buf.append, res)

		log = AssertHandler()

		thr.start()
		time.sleep(delay.value * 4.5)
		thr.done = True

		log.assert_logged('error', 'not readable')

		eq_(buf, [])


if __name__ == '__main__':
	main()



from nose.tools import assert_raises, eq_
from pickle import dumps, loads
from unittest import main

from spacq.tests.tool.box import DeviceServerTestCase
from ...abstract_device import AbstractDevice

from ... import config


class DeviceConfigTest(DeviceServerTestCase):
	def obtain_device(self):
		"""
		Get a real device with which to test.
		"""

		return DeviceServerTestCase.obtain_device(self)

	def populate_config(self, cfg, addr):
		"""
		Given an address dictionary, populate the DeviceConfig.
		"""

		if 'ip_address' in addr:
			cfg.address_mode = cfg.address_modes.ethernet
			cfg.ip_address = addr['ip_address']
		elif 'gpib_pad' in addr:
			cfg.address_mode = cfg.address_modes.gpib
			try:
				cfg.gpib_board = addr['gpid_board']
			except KeyError:
				pass
			cfg.gpib_pad = addr['gpib_pad']
			try:
				cfg.gpib_sad = addr['gpid_sad']
			except KeyError:
				pass
		elif 'usb_resource' in addr:
			cfg.address_mode = cfg.address_modes.usb
			cfg.usb_resource = addr['usb_resource']

	def testConnectSuccessful(self):
		"""
		Connect normally.
		"""

		dev = self.obtain_device()

		cfg = config.DeviceConfig(name='Test')
		self.populate_config(cfg, dev['address'])

		cfg.manufacturer = dev['manufacturer']
		cfg.model = dev['model']

		cfg.connect()

		assert isinstance(cfg.device, AbstractDevice)

		return cfg

	def testConnectNoImplementation(self):
		"""
		Try to connect without an implementation.
		"""

		dev = self.obtain_device()

		cfg = config.DeviceConfig(name='Test')
		self.populate_config(cfg, dev['address'])

		assert_raises(config.ConnectionError, cfg.connect)

	def testConnectNoAddress(self):
		"""
		Try to connect without an address.
		"""

		dev = self.obtain_device()

		cfg = config.DeviceConfig(name='Test')

		cfg.manufacturer = dev['manufacturer']
		cfg.model = dev['model']

		assert_raises(config.ConnectionError, cfg.connect)

	def testPickle(self):
		"""
		Pickle and unpickle a device config.
		"""

		cfg = self.testConnectSuccessful()

		expected = cfg.__dict__.copy()
		del expected['_device']
		del expected['resources']

		actual = loads(dumps(cfg)).__dict__
		del actual['_device']
		del actual['resources']

		eq_(actual, expected)


if __name__ == '__main__':
	main()



from nose.tools import eq_
from unittest import main

from spacq.tests.tool.box import DeviceServerTestCase

from ... import dm34410a


class DM34410ATest(DeviceServerTestCase):
	def obtain_device(self):
		dev = DeviceServerTestCase.obtain_device(self, impl=dm34410a.DM34410A,
				manufacturer='Agilent', model='34410A')
		dev.reset()

		return dev

	def testAutoZero(self):
		"""
		Test the auto zero setting.
		"""

		dm = self.obtain_device()

		dm.auto_zero = 'once'
		eq_(dm.auto_zero, 'off')

		dm.auto_zero = 'on'
		eq_(dm.auto_zero, 'on')

		try:
			dm.auto_zero = 'something else'
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'

	def testIntegrationTime(self):
		"""
		Test the integration time setting.
		"""

		dm = self.obtain_device()

		dm.integration_time = 100
		eq_(dm.integration_time, 100)

		try:
			dm.integration_time = -999
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'

	def testGetValues(self):
		"""
		Obtain some values.
		"""

		dm = self.obtain_device()

		dm.reading.assert_dimensions('V')


if __name__ == '__main__':
	main()



from math import pi, sin
from random import randint
from struct import pack

from ...mock.mock_abstract_device import MockAbstractDevice
from ...tools import BlockData
from ..dpo7104 import DPO7104

"""
Mock Tektronix DPO7104 Digital Phosphor Oscilloscope

Control a fake DPO's settings and input waveforms.
"""


class MockChannel(object):
	"""
	A mock channel for a mock DPO.
	"""

	def __init__(self):
		self.enabled = False
		self.scale = 0.5
		self.offset = 0.0


class MockDPO7104(MockAbstractDevice, DPO7104):
	"""
	Mock interface for Tektronix DPO7104 DPO.
	"""

	def __init__(self, *args, **kwargs):
		self.mocking = DPO7104

		MockAbstractDevice.__init__(self, *args, **kwargs)

	def _reset(self):
		self.mock_state['stopafter'] = 'runstop'
		self.mock_state['acquire_state'] = True

		self.mock_state['fastframe'] = False
		self.mock_state['fastframe_sum'] = 'none'
		self.mock_state['fastframe_count'] = 1

		self.mock_state['samplerate'] = 1e10 # Hz
		self.mock_state['horizontal_scale'] = 1e-8 # s

		self.mock_state['waveform_bytes'] = 2

		self.mock_state['data_start'] = 1
		self.mock_state['data_stop'] = self._record_length
		self.mock_state['data_source'] = 1
		self.mock_state['data_framestart'] = 1
		self.mock_state['data_framestop'] = 1

		self.mock_state['channels'] = [None] # There is no channel 0.
		for _ in xrange(1, 5):
			self.mock_state['channels'].append(MockChannel())
		self.mock_state['channels'][1].enabled = True

	@property
	def _record_length(self):
		return int(10 * self.mock_state['samplerate'] * self.mock_state['horizontal_scale'])

	def write(self, message, result=None, done=False):
		if not done:
			cmd, args, query = self._split_message(message)

			if cmd[0] == 'autoset':
				if args == 'execute':
					done = True
			elif cmd[0] == 'acquire':
				if cmd[1] == 'stopafter':
					if query:
						result = self.mock_state['stopafter']
					else:
						self.mock_state['stopafter'] = args
					done = True
				elif cmd[1] == 'state':
					if query:
						result = str(int(self.mock_state['acquire_state']))
					else:
						self.mock_state['acquire_state'] = bool(int(args))
					done = True
				elif cmd[1] == 'mode':
					if query:
						result = self.mock_state['acquire_mode']
					else:
						self.mock_state['acquire_mode'] = args
					done = True
				elif cmd[1] == 'numacq' and query:
					result = self.mock_state['fastframe_count']
					done = True
			elif cmd[0] == 'horizontal':
				if cmd[1] == 'mode':
					if cmd[2] == 'samplerate':
						if query:
							result = self.mock_state['samplerate']
						else:
							self.mock_state['samplerate'] = float(args)
						done = True
					elif cmd[2] == 'scale':
						if query:
							result = self.mock_state['horizontal_scale']
						else:
							self.mock_state['horizontal_scale'] = float(args)
						done = True
					elif cmd[2] == 'recordlength' and query:
						result = self._record_length
						done = True
				elif cmd[1] == 'divisions' and query:
					result = 10
					done = True
				elif cmd[1] == 'fastframe':
					if cmd[2] == 'state':
						if query:
							result = int(self.mock_state['fastframe'])
						else:
							self.mock_state['fastframe'] = bool(int(args))
						done = True
			elif cmd[0] == 'data':
				if cmd[1] == 'start':
					if query:
						result = self.mock_state['data_start']
					else:
						self.mock_state['data_start'] = int(args)
					done = True
				elif cmd[1] == 'stop':
					if query:
						result = self.mock_state['data_stop']
					else:
						self.mock_state['data_stop'] = int(args)
					done = True
				elif cmd[1] == 'source':
					if query:
						result = 'ch{0}'.format(self.mock_state['data_source'])
					else:
						self.mock_state['data_source'] = int(args[2])
					done = True
				if cmd[1] == 'framestart':
					if query:
						result = self.mock_state['data_framestart']
					else:
						self.mock_state['data_framestart'] = int(args)
					done = True
				elif cmd[1] == 'framestop':
					if query:
						result = self.mock_state['data_framestop']
					else:
						self.mock_state['data_framestop'] = int(args)
					done = True
			elif cmd[0] == 'curve' and query:
				num_points = self._record_length * self.mock_state['waveform_bytes']
				ch = self.mock_state['data_source']
				curve = [int(120 * sin(2 * ch * pi * x / num_points) + randint(-7, 7)) for x in xrange(num_points)]
				result = BlockData.to_block_data(pack('!%db' % (num_points), *curve))
				done = True
			elif cmd[0] == 'wfmoutpre':
				if cmd[1] == 'byt_nr':
					if query:
						result = self.mock_state['waveform_bytes']
					else:
						self.mock_state['waveform_bytes'] = int(args)
					done = True
			elif cmd[0] == 'select':
				if cmd[1].startswith('ch'):
					channel = int(cmd[1][2])

					if query:
						result = str(int(self.mock_state['channels'][channel].enabled))
					else:
						self.mock_state['channels'][channel].enabled = (args == 'on')
					done = True
			elif cmd[0].startswith('ch'):
				channel = int(cmd[0][2])

				if cmd[1] == 'scale':
					if query:
						result = self.mock_state['channels'][channel].scale
					else:
						self.mock_state['channels'][channel].scale = float(args)
					done = True
				elif cmd[1] == 'offset':
					if query:
						result = self.mock_state['channels'][channel].offset
					else:
						self.mock_state['channels'][channel].offset = float(args)
					done = True

		MockAbstractDevice.write(self, message, result, done)


name = 'DPO7104'
implementation = MockDPO7104



import logging
log = logging.getLogger(__name__)

from nose.tools import eq_
from numpy import linspace
from numpy.testing import assert_array_almost_equal
from unittest import main

from spacq.interface.units import Quantity
from spacq.tests.tool.box import AssertHandler, DeviceServerTestCase

from ... import awg5014b


class AWG5014BTest(DeviceServerTestCase):
	def obtain_device(self):
		return DeviceServerTestCase.obtain_device(self, impl=awg5014b.AWG5014B,
				manufacturer='Tektronix', model='AWG5014B')

	def testMarkerValues(self):
		"""
		Set the various marker values.
		"""

		awg = self.obtain_device()
		awg.reset()

		awg.channels[1].markers[1].delay = Quantity(1, 'ns')
		awg.channels[1].markers[1].high = Quantity(0.5, 'V')
		awg.channels[1].markers[2].delay = Quantity(0.1, 'ns')
		awg.channels[2].markers[1].low = Quantity(-100, 'mV')

		eq_(awg.channels[1].markers[1].delay.value, 1e-9)
		eq_(awg.channels[1].markers[2].delay.value, 0.1e-9)
		eq_(awg.channels[2].markers[1].delay.value, 0)
		eq_(awg.channels[2].markers[2].delay.value, 0)

		eq_(awg.channels[1].markers[1].high.value, 0.5)
		eq_(awg.channels[1].markers[2].high.value, 1)
		eq_(awg.channels[2].markers[1].high.value, 1)
		eq_(awg.channels[2].markers[2].high.value, 1)

		eq_(awg.channels[1].markers[1].low.value, 0)
		eq_(awg.channels[1].markers[2].low.value, 0)
		eq_(awg.channels[2].markers[1].low.value, -0.1)
		eq_(awg.channels[2].markers[2].low.value, 0)

	def testScenario(self):
		"""
		Run through a simple scenario.

		Note: Verification should also be done manually based on the AWG output.
		"""

		log = AssertHandler()

		awg = self.obtain_device()
		awg.reset()

		assert not awg.enabled

		# Setup
		existing_waveforms = awg.waveform_names

		data1 = linspace(-1.0, 1.0, 21)
		data2 = linspace(1.0, -1.0, 21)

		log.flush()
		awg.channels[1].set_waveform(data1, {
			1: ([1, 1, 1, 0, 0] * len(data1))[:len(data1)],
			2: ([0, 0, 0, 1, 1] * len(data1))[:len(data1)],
			3: [1, 2, 3, 4],
		})
		log.assert_logged('warning', 'marker 3 ignored: \[1, 2, 3, 4\]')

		awg.channels[2].set_waveform(data2, name='Test 2')

		awg.sampling_rate = Quantity(200, 'MHz')

		awg.channels[1].enabled = True
		awg.channels[1].amplitude = Quantity(0.8, 'V')

		awg.channels[2].enabled = True
		awg.channels[2].amplitude = Quantity(0.4, 'V')

		awg.channels[3].waveform_name = 'Test 2'
		awg.channels[3].enabled = True

		awg.channels[4].waveform_name = 'Channel 1'

		del awg.channels[3].waveform_name

		awg.run_mode = 'triggered'
		awg.enabled = True

		# Verify
		eq_(awg.sampling_rate.value, 2e8)

		eq_(awg.waveform_names, existing_waveforms + ['Channel 1', 'Test 2'])

		assert_array_almost_equal(awg.get_waveform('Channel 1'), data1, 4)
		eq_(awg.channels[1].amplitude.value, 0.8)
		assert_array_almost_equal(awg.get_waveform('Test 2'), data2, 4)
		eq_(awg.channels[2].amplitude.value, 0.4)

		for ch in [1, 2]:
			eq_(awg.channels[ch].enabled, True)
		for ch in [3, 4]:
			eq_(awg.channels[ch].enabled, False)

		for ch in [1, 4]:
			eq_(awg.channels[ch].waveform_name, 'Channel 1')
		eq_(awg.channels[2].waveform_name, 'Test 2')
		eq_(awg.channels[3].waveform_name, '')

		eq_(awg.run_mode, 'triggered')
		assert awg.waiting_for_trigger
		assert awg.enabled

		awg.trigger()

		assert awg.waiting_for_trigger
		assert awg.enabled

		awg.run_mode = 'continuous'

		assert not awg.waiting_for_trigger
		assert awg.enabled


if __name__ == '__main__':
	main()



from nose.tools import assert_raises, eq_
from unittest import main, TestCase

from spacq.interface.units import IncompatibleDimensions, Quantity

from .. import variables


class SortVariablesTest(TestCase):
	def testEmpty(self):
		"""
		Use no variables.
		"""

		sorted_variables, num_items = variables.sort_variables([])

		eq_(sorted_variables, [])
		eq_(num_items, 0)

	def testSingle(self):
		"""
		Use a single variable.
		"""

		var = variables.OutputVariable(config=variables.LinSpaceConfig(-5.0, 5.0, 11),
				name='Name', order=0, enabled=True, const=60.0)

		sorted_variables, num_items = variables.sort_variables([var])

		eq_(sorted_variables, [(var,)])
		eq_(num_items, 11)

	def testMultiple(self):
		"""
		Use many variables.
		"""

		vars = [
			variables.OutputVariable(config=variables.LinSpaceConfig(1.0, 5.0, 3),
					name='A', order=3, enabled=True),
			variables.OutputVariable(config=variables.LinSpaceConfig(11.0, 12.0, 2),
					name='B', order=2, enabled=True, const=10.0),
			variables.OutputVariable(config=variables.LinSpaceConfig(-99.0, 0.0),
					name='D', order=1, enabled=True, const=9.0, use_const=True),
			variables.OutputVariable(config=variables.LinSpaceConfig(21.0, 25.0, 20),
					name='C', order=2, enabled=True),
			variables.OutputVariable(config=variables.LinSpaceConfig(0.0, 0.0, 1),
					name='E', order=4),
			variables.OutputVariable(
					name='F', order=5, enabled=True, const=5.5, use_const=True),
		]

		sorted_variables, num_items = variables.sort_variables(vars)

		eq_(sorted_variables, [(vars[2], vars[5]), (vars[0],), (vars[1], vars[3])])
		eq_(num_items, 6)


class OutputVariableTest(TestCase):
	def testAdjust(self):
		"""
		Try to adjust the values after initialization.
		"""

		var = variables.OutputVariable(name='Name', order=1)

		var.config = variables.LinSpaceConfig()

		var.config.steps = 1000
		eq_(var.config.steps, 1000)

		try:
			var.config.steps = -1
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'

		var.wait = '1e2 ms'
		eq_(var.wait, '100 ms')

		try:
			var.wait = '100 Hz'
		except IncompatibleDimensions:
			pass
		else:
			assert False, 'Expected IncompatibleDimensions.'

	def testStr(self):
		"""
		Ensure the variable looks right.
		"""

		var = variables.OutputVariable(name='Name', order=1)

		# Very short.
		var.config = variables.LinSpaceConfig(0.0, 5.0, 3)
		var.type = 'integer'
		eq_(str(var), '[0, 2, 5]')

		# Borderline.
		var.config = variables.LinSpaceConfig(1.0, 5.0, 5)
		var.type = 'float'
		eq_(str(var), '[1, 2, 3, 4, 5]')

		# Short enough.
		var.config = variables.LinSpaceConfig(-200.0, 200.0, 401)
		eq_(str(var), '[-200, -199, -198, -197, ..., 200]')

		# Far too long.
		var.config = variables.LinSpaceConfig(0.0, 100000.0, 100001)
		eq_(str(var), '[0, 1, 2, 3, ...]')

		# Smooth from constant.
		var.smooth_from = True
		eq_(str(var), '(0, 1, 2, 3, ...]')

		# And to.
		var.smooth_to = True
		eq_(str(var), '(0, 1, 2, 3, ...)')

	def testUnits(self):
		"""
		Ensure that values are wrapped with units.
		"""

		var = variables.OutputVariable(name='Name', order=1)
		var.type = 'quantity'
		var.units = 'g.m.s-1'
		var.config = variables.LinSpaceConfig(0.0, -5.0, 3)

		eq_(list(var), [Quantity(x, 'g.m.s-1') for x in [0, -2.5, -5]])
		eq_(str(var), '[0, -2.5, -5] g.m.s-1')

		# Bad combination.
		var.units = None
		assert_raises(ValueError, list, var)


class LinSpaceConfigTest(TestCase):
	def testIterator(self):
		"""
		Create an iterator from a linear space variable.
		"""

		var = variables.OutputVariable(config=variables.LinSpaceConfig(-1.0, -3.0, 5),
				name='Name', order=1, enabled=True, const=10.0)

		# Non-const.
		eq_(len(var), 5)

		it1 = iter(var)
		eq_(list(it1), [-1.0, -1.5, -2.0, -2.5, -3.0])

		# Const.
		var.use_const = True

		eq_(len(var), 1)

		it2 = iter(var)
		eq_(list(it2), [10.0])


class ArbitraryConfigTest(TestCase):
	def testIterator(self):
		"""
		Create an iterator from an arbitrary variable.
		"""

		values = [8, -5, 6.6, 3, 0, 0]

		var = variables.OutputVariable(config=variables.ArbitraryConfig(values),
				name='Name', order=1, enabled=True)

		eq_(len(var), len(values))

		it = iter(var)
		eq_(list(it), values)


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

import numpy
import time

from spacq.interface.resources import Resource
from spacq.interface.units import Quantity
from spacq.tool.box import Synchronized

from ..abstract_device import AbstractDevice, AbstractSubdevice
from ..tools import quantity_unwrapped, BinaryEncoder

"""
Custom voltage source

Control the output voltages on all the ports.
"""


class Port(AbstractSubdevice):
	"""
	An output port on the voltage source.
	"""

	# Since there is no way to determine whether calibration has completed,
	# wait this long and hope for the best.
	calibration_delay = 2 # s

	@staticmethod
	def format_for_dac(msg):
		"""
		Perform some formatting to make the device happy:
			flip all the bits in the message
			pad messages until their length in bytes is a multiple of 4
		"""

		log.debug('Formatting for DAC: {0}'.format(msg))

		msg_encoded = BinaryEncoder.encode(msg)
		# Flip each byte separately.
		msg_flipped = [chr(~ord(x) & 0xff) for x in msg_encoded]

		missing_bytes = (4 - len(msg_encoded) % 4) % 4

		result = BinaryEncoder.decode(msg_flipped + ['\x00'] * missing_bytes)

		log.debug('Formatted for DAC (padded with {0} bytes): {1}'.format(missing_bytes, result))

		return result

	def _setup(self):
		AbstractSubdevice._setup(self)

		# These values are used to tune the input values according to empirical error.
		self.gain = 1.0
		self.offset = 0.0

		# Resources.
		write_only = ['voltage']
		for name in write_only:
			self.resources[name] = Resource(self, None, name)

		self.resources['voltage'].units = 'V'

	@Synchronized()
	def _connected(self):
		AbstractSubdevice._connected(self)

		if self.do_apply_settings:
			self.apply_settings(calibrate=False)

	def __init__(self, device, num, resolution=20, apply_settings=True, min_value=-10,
			max_value=+10, adaptive_filtering=True, calibrate_connected=False,
			fast_settling=True,	freq=100, *args, **kwargs):
		"""
		Initialize the output port.

		device: The VoltageSource to which this Port belongs.
		num: The index of this port.
		resolution: How many bits the output value contains.
		apply_settings: Whether to automatically apply all the settings.
		min_value: Smallest value the port can produce.
		max_value: Largest value the port can produce.
		adaptive_filtering: Enable adaptive filtering.
		calibrate_connected: Do not disconnect output while calibrating.
		fast_settling: Enable fast settling.
		freq: Clock rate in kHz.
		"""

		AbstractSubdevice.__init__(self, device, *args, **kwargs)

		if resolution not in [16, 20]:
			raise ValueError('Unsupported resolution: {0}'.format(resolution))

		self.num = num
		self.resolution = resolution
		self.do_apply_settings = apply_settings
		self.min_value = min_value
		self.max_value = max_value
		self.adaptive_filtering = adaptive_filtering
		self.calibrate_connected = calibrate_connected
		self.fast_settling = fast_settling
		self.freq = freq

	def calculate_voltage(self, voltage):
		"""
		Determine the value corresponding to the given voltage.
		"""

		try:
			voltage_adjusted = voltage * self.gain + self.offset
		except TypeError:
			raise ValueError('Voltage must be a number. Given: {0}'.format(voltage))

		if voltage_adjusted < self.min_value or voltage_adjusted > self.max_value:
			raise ValueError('Adjusted voltage must be within [{0}, {1}]. '
					'Given: {2}; adjusted to: {3}.'.format(self.min_value,
					self.max_value, voltage, voltage_adjusted))

		max_converted = (1 << self.resolution) - 1
		value_span = self.max_value - self.min_value

		# Map [-min_value, max_value] onto [0x0, 0xff...] depending on the resolution.
		# First negate the voltage, so that flipping the bits later will make it correct.
		return int(float(-voltage_adjusted + self.max_value) / value_span * max_converted)

	@Synchronized()
	def write_to_dac(self, message):
		"""
		Write a message to the DAC of the port.

		Voodoo programming follows, thanks to:
			NI's lack of support for anything non-Windows in this case
			my lack of time & desire to properly reverse engineer the ni845x DLL

		If the conversation does not go according to plan, bails out with an AssertionError!
		"""

		message_length = BinaryEncoder.length(message)

		if message_length > 4:
			raise ValueError('Message is longer than 4 bytes: {0}'.format(message))

		message_formatted = self.format_for_dac(message)

		# The reply always comes back with as many bits set to 1 as were sent.
		expected_reply = self.format_for_dac('00' * message_length)

		# Lots of assertions so we can bail ASAP to avoid crashing anything.
		self.device.ask_encoded('0000 000c 0008 0100 0000 0000',
				'0000 001c 0018 0100 0000 0002 0200 1000 0100 c001 0100 c000 0002 0000')
		self.device.ask_encoded('0000 0010 000c 0113 0280 0000 0000 ff01',
				'0000 000c 0008 0100 0000 0002')
		self.device.ask_encoded('0000 0010 000c 0112 0280 0000 00ff ff00',
				'0000 000c 0008 0100 0000 0002')
		self.device.ask_encoded('0000 0010 000c 0111 0280 0000 00ff {0:02x} 00'.format(self.num),
				'0000 000c 0008 0100 0000 0002')
		self.device.ask_encoded('0000 000c 0008 0100 0000 0000',
				'0000 001c 0018 0100 0000 0002 0200 1000 0100 c001 0100 c000 0002 0000')
		self.device.ask_encoded('0000 0014 0010 0110 0260 0000 0000 {0:04x} 0700 0000'.format(self.freq),
				'0000 000c 0008 0100 0000 0002')
		self.device.ask_encoded('0000 0014 0010 0111 0260 0000 0003 {0:02x} 00 {1}'.format(message_length,
				message_formatted),
				'0000 0014 0010 0100 0000 0002 {0:02x} 00 0000 {1}'.format(message_length, expected_reply))

	def apply_settings(self, calibrate=False):
		"""
		Apply the settings for the DAC on this port.

		calibrate: Run self-calibration on this port as well.

		Note: If self-calibrating, it is essential to wait the calibration_delay after this method returns.
		"""

		flags = ((not self.adaptive_filtering) << 15 |
				self.calibrate_connected << 14 |
				(not self.fast_settling) << 4)

		if calibrate:
			flags |= 0b01

		# Write 16 bits to the top of the DIR: 0010 0100 xx10 0000 101x 00xx
		self.write_to_dac('24 {0:04x}'.format(0x20a0 | flags))

	@quantity_unwrapped('V')
	def set_voltage(self, voltage):
		"""
		Set the voltage on this port, as a quantity in V.
		"""

		# Left-align the bits within the value:
		# 20-bit: VVVV VVVV VVVV VVVV VVVV xxxx
		# 16-bit: VVVV VVVV VVVV VVVV xxxx xxxx
		# where the 'x's are don't-cares, so we just set them to 0 by shifting.
		resulting_voltage = self.calculate_voltage(voltage) << (24 - self.resolution)

		# Write 24 bits to the top of the DIR: 0100 0000 xxxx xxxx xxxx xxxx xxxx xxxx
		self.write_to_dac('40 {0:06x}'.format(resulting_voltage))

	voltage = property(fset=set_voltage)

	@Synchronized()
	def autotune(self, voltage_resource, min_value=None, max_value=None, final_value=0, set_result=True):
		"""
		Take some measured data and solve for the gain and offset.

		voltage_resource: A resource which provides the realtime measured data for this port.
		min_value: Smallest value to take into account.
		max_value: Largest value to take into account.
		final_value: Value to set port to after all measurements are taken.
		set_result: Whether to apply the resulting gain and offset.
		"""

		self.device.status.append('Autotuning port {0}'.format(self.num))

		try:
			if min_value is None:
				min_value = self.min_value
			if max_value is None:
				max_value = self.max_value

			# Test with raw values.
			old_gain, old_offset = self.gain, self.offset
			self.gain, self.offset = 1, 0

			if max_value < min_value:
				raise ValueError('{0} > {1}'.format(min_value, max_value))
			elif max_value == min_value:
				num_points = 1
			else:
				num_points = 21

			# Obtain data.
			real = numpy.linspace(min_value, max_value, num_points)
			measured = []

			for x in real:
				self.voltage = Quantity(x, 'V')
				time.sleep(0.2)
				measured.append(voltage_resource.value.value)

			# Solve.
			A = numpy.vstack([measured, numpy.ones(len(measured))]).T
			gain, offset = numpy.linalg.lstsq(A, real)[0]

			if set_result:
				self.gain, self.offset = gain, offset
			else:
				self.gain, self.offset = old_gain, old_offset

			# Set the voltage after the gain and offset, so that it is potentially more correct.
			self.voltage = Quantity(final_value, 'V')

			return (gain, offset)
		finally:
			self.device.status.pop()


class VoltageSource(AbstractDevice):
	"""
	Interface for the custom voltage source.

	It uses several TI DAC1220 chips and an NI USB-8451 to interface with them over SPI.
	"""

	@property
	def _gui_setup(self):
		try:
			from .gui.voltage_source import VoltageSourceSettingsDialog

			return VoltageSourceSettingsDialog
		except ImportError as e:
			log.debug('Could not load GUI setup for device "{0}": {1}'.format(self.name, str(e)))

			return None

	def _setup(self):
		AbstractDevice._setup(self)

		self.ports = []
		for num in xrange(16):
			port = Port(self, num, **self.port_settings)
			self.ports.append(port)
			self.subdevices['port{0:02}'.format(num)] = port

	def __init__(self, port_settings=None, *args, **kwargs):
		"""
		Initialize the voltage source and all its ports.

		port_settings: A dictionary of values to give to each port upon creation.
		"""

		if port_settings is None:
			self.port_settings = {}
		else:
			self.port_settings = port_settings

		AbstractDevice.__init__(self, *args, **kwargs)

	@Synchronized()
	def ask_encoded(self, msg, assertion=None):
		"""
		Encode and write the message; then read and decode the answer.
		"""

		self.write(BinaryEncoder.encode(msg))
		result = BinaryEncoder.decode(self.read_raw())

		if assertion is not None:
			# Ensure that extra formatting doesn't trigger an assertion failure.
			formatted_assertion = BinaryEncoder.decode(BinaryEncoder.encode(assertion))

			assert result == formatted_assertion, (
					'Device in unknown state; expect general failure. '
					'Asserted: {0}; observed: {1}.'.format(assertion, result))

		return result


name = 'Voltage source'
implementation = VoltageSource



from unittest import main

from ... import dm34410a
from .. import mock_dm34410a

from ...tests.server.test_dm34410a import DM34410ATest


# Don't lose the real device.
real_DM34410A = dm34410a.DM34410A
is_mock = DM34410ATest.mock


def setup():
	# Run the tests with a fake device.
	dm34410a.DM34410A = mock_dm34410a.MockDM34410A
	DM34410ATest.mock = True

def teardown():
	# Restore the real device for any remaining tests.
	dm34410a.DM34410A = real_DM34410A
	DM34410ATest.mock = is_mock


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

from functools import wraps
import string

from spacq.interface.units import Quantity

"""
Tools for working with hardware devices.
"""


def str_to_bool(value):
	"""
	False and 'False' => False
	otherwise => True
	"""

	return bool(value) and value.lower() != 'false'


def quantity_wrapped(units, multiplier=1.0):
	"""
	A decorator for getters to wrap the plain device value into a quantity with a unit.
	"""

	def wrap(f):
		@wraps(f)
		def wrapped(self):
			return Quantity(f(self) * multiplier, units)

		return wrapped

	return wrap

def quantity_unwrapped(units, multiplier=1.0):
	"""
	A decorator for setters to extract the plain device value from the quantity.
	"""

	def wrap(f):
		@wraps(f)
		def wrapped(self, value):
			value.assert_dimensions(units)

			return f(self, value.value * multiplier)

		return wrapped

	return wrap


class BlockDataError(Exception):
	"""
	Problem reading block data.
	"""

	pass


class BlockData(object):
	"""
	Utility methods for conversion between binary and 488.2 block data.
	"""

	@staticmethod
	def to_block_data(data):
		"""
		Packs binary data into 488.2 block data.

		As per section 7.7.6 of IEEE Std 488.2-1992.

		Note: Does not produce indefinitely-formatted block data.
		"""

		log.debug('Converting to block data: {0!r}'.format(data))

		length = len(data)
		length_length = len(str(length))

		return '#{0}{1}{2}'.format(length_length, length, data)

	@staticmethod
	def from_block_data(block_data):
		"""
		Extracts binary data from 488.2 block data.

		As per section 7.7.6 of IEEE Std 488.2-1992.
		"""

		log.debug('Converting from block data: {0!r}'.format(block_data))

		# Must have at least "#0\n" or "#XX".
		if len(block_data) < 3:
			raise BlockDataError('Not enough data.')

		if block_data[0] != '#':
			raise BlockDataError('Leading character is "{0}", not "#".'.format(block_data[0]))

		if block_data[1] == '0':
			log.debug('Indefinite format.')

			if block_data[-1] != '\n':
				raise BlockDataError('Final character is "{0}", not NL.'.format(block_data[-1]))

			return block_data[2:-1]
		else:
			log.debug('Definite format.')

			try:
				length_length = int(block_data[1])
			except ValueError:
				raise BlockDataError('Length length incorrectly specified: {0}'.format(block_data[1]))

			data_start = 2 + length_length

			if data_start > len(block_data):
				raise BlockDataError('Not enough data.')

			try:
				length = int(block_data[2:data_start])
			except ValueError:
				raise BlockDataError('Length incorrectly specified: {0}'.format(block_data[2:data_start]))

			data_end = data_start + length

			if data_end > len(block_data):
				raise BlockDataError('Not enough data.')
			elif data_end < len(block_data):
				if block_data[data_end:] != '\n':
					log.warning('Extra data ignored: {0!r}'.format(block_data[data_end:]))

			return block_data[data_start:data_end]


class BinaryEncoder(object):
	"""
	Utility methods for dealing with encoding and decoding binary data.
	"""

	@staticmethod
	def encode(msg):
		"""
		Convert a string of hexadecimal digits to a byte string.
		"""

		log.debug('Encoding to byte string: {0}'.format(msg))

		# Discard non-hexadecimal characters.
		msg_filtered = [x for x in msg if x in string.hexdigits]
		# Grab pairs.
		idxs = xrange(0, len(msg_filtered), 2)
		msg_paired = [''.join(msg_filtered[i:i+2]) for i in idxs]
		# Convert to bytes.
		msg_encoded = ''.join([chr(int(x, 16)) for x in msg_paired])

		log.debug('Encoded to: {0!r}'.format(msg_encoded))

		return msg_encoded

	@staticmethod
	def decode(msg, pair_size=2, pair_up=True):
		"""
		Convert a byte string to a string of hexadecimal digits.
		"""

		log.debug('Decoding from byte string: {0!r}'.format(msg))

		# Get the hex string for each byte.
		msg_decoded = ['{0:02x}'.format(ord(x)) for x in msg]

		if pair_up:
			idxs = xrange(0, len(msg_decoded), pair_size)
			msg_formatted = [''.join(msg_decoded[i:i+pair_size]) for i in idxs]

			result = ' '.join(msg_formatted)
		else:
			result = ''.join(msg_decoded)

		log.debug('Decoded to: {0}'.format(result))

		return result

	@staticmethod
	def length(msg):
		"""
		Calculate the number of bytes an unencoded message takes up when encoded.
		"""

		log.debug('Finding encoded length: {0}'.format(msg))

		result = len(BinaryEncoder.encode(msg))

		log.debug('Found encoded length: {0}'.format(result))

		return result



from nose.tools import assert_almost_equal, eq_
from unittest import main

from spacq.interface.units import Quantity
from spacq.tests.tool.box import DeviceServerTestCase

from ... import smf100a


class SMF100ATest(DeviceServerTestCase):
	def obtain_device(self):
		return DeviceServerTestCase.obtain_device(self, impl=smf100a.SMF100A,
				manufacturer='Rohde & Schwarz', model='SMF100A')

	def testScenario(self):
		"""
		Change the settings.
		"""

		sg = self.obtain_device()
		sg.reset()

		sg.write('unit:power v')

		assert not sg.enabled
		eq_(sg.frequency.value, 1e10)
		assert_almost_equal(sg.power.value, 0.007, 3)

		sg.frequency = Quantity(5.6789, 'GHz')
		sg.power = Quantity(100, 'mV')
		sg.enabled = True

		assert sg.enabled
		eq_(sg.frequency.value, 5.6789e9)
		eq_(sg.power.value, 0.1)

		sg.enabled = False
		assert not sg.enabled

	def testIllegal(self):
		"""
		These values aren't allowed.
		"""

		sg = self.obtain_device()

		sg.write('unit:power v')

		try:
			sg.power = Quantity(9.001, 'V')
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError'

		try:
			sg.frequency = Quantity(0, 'Hz')
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError'


if __name__ == '__main__':
	main()



from nose.tools import eq_
from unittest import main, TestCase

from .. import voltage_source


class PortTest(TestCase):
	def testCalculateVoltage(self):
		"""
		Trivial conversion.
		"""

		port = voltage_source.Port(None, None, apply_settings=False, resolution=20)

		data = [
			(-10, 0xfffff),
			(-5, 0xbffff),
			(0, 0x7ffff),
			(5, 0x3ffff),
			(10, 0x00000),
		]

		for v, r in data:
			eq_(port.calculate_voltage(v), r)

	def testResolution(self):
		"""
		Trivial conversion at another resolution.
		"""

		port = voltage_source.Port(None, None, apply_settings=False, resolution=16)

		data = [
			(-10, 0xffff),
			(-5, 0xbfff),
			(0, 0x7fff),
			(5, 0x3fff),
			(10, 0x0000),
		]

		for v, r in data:
			eq_(port.calculate_voltage(v), r)

	def testVoltageBounds(self):
		"""
		Trivial conversion gone wrong.
		"""

		port = voltage_source.Port(None, None, apply_settings=False)

		data = ['x', -10.1, 10.1, None, 42]

		for v in data:
			try:
				port.calculate_voltage(v)
			except ValueError:
				pass
			else:
				assert False, 'Expected ValueError'

	def testFormatForDAC(self):
		"""
		Make the messages more palatable.
		"""

		data = [
			('', ''),
			('0', 'ff00 0000'),
			('00', 'ff00 0000'),
			('0000 0000', 'ffff ffff'),
			('ffff', '0000 0000'),
			('123456', 'edcb a900'),
		]

		for m, f in data:
			eq_(voltage_source.Port.format_for_dac(m), f)


	def testCustomBounds(self):
		"""
		Try conversions with custom minimum and maximum values.
		"""

		port = voltage_source.Port(None, None, apply_settings=False, min_value=-100, max_value=-30)

		data = [
			(-100, 0xfffff),
			(-65, 0x7ffff),
			(-30.00007, 0x00001),
			(-30, 0x00000),
		]

		for v, r in data:
			eq_(port.calculate_voltage(v), r)


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)


name = 'Rohde & Schwarz'

from . import smf100a
models = [smf100a]
log.debug('Found models for "{0}": {1}'.format(name, ''.join(str(x) for x in models)))

from .mock import mock_smf100a
mock_models = [mock_smf100a]
log.debug('Found mock models for "{0}": {1}'.format(name, ''.join(str(x) for x in mock_models)))



import logging
log = logging.getLogger(__name__)

from spacq.interface.resources import Resource
from spacq.tool.box import Synchronized

from ..abstract_device import AbstractDevice
from ..tools import str_to_bool, quantity_wrapped, quantity_unwrapped

"""
Interface for R&S SMF100A signal generator.
"""


class SMF100A(AbstractDevice):
	"""
	Interface for the SMF100A.
	"""

	min_power = 0.007 # V
	max_power = 7.071 # V

	min_freq = 1e9 # Hz
	max_freq = 22e9 # Hz

	def _setup(self):
		AbstractDevice._setup(self)

		# Resources.
		read_write = ['enabled', 'power', 'frequency']
		for name in read_write:
			self.resources[name] = Resource(self, name, name)

		self.resources['enabled'].converter = str_to_bool
		self.resources['power'].units = 'V'
		self.resources['frequency'].units = 'Hz'

	@Synchronized()
	def _connected(self):
		AbstractDevice._connected(self)

		# Set the units for communication.
		self.write('unit:power v')

	@Synchronized()
	def reset(self):
		log.info('Resetting "{0}".'.format(self.name))
		self.write('*rst')

	@property
	def enabled(self):
		"""
		Whether the RF output is enabled.
		"""

		return bool(int(self.ask('output:state?')))

	@enabled.setter
	def enabled(self, value):
		self.write('output:state {0}'.format(int(value)))

	@property
	@quantity_wrapped('V')
	def power(self):
		"""
		The RF output power, as a quantity in V.
		"""

		return float(self.ask('source:power:power?'))

	@power.setter
	@quantity_unwrapped('V')
	def power(self, value):
		if value < self.min_power or value > self.max_power:
			raise ValueError('Value {0} not within the allowed bounds: {1} to {2}'.format(value,
				self.min_power, self.max_power))

		self.write('source:power:power {0}'.format(value))

	@property
	@quantity_wrapped('Hz')
	def frequency(self):
		"""
		The RF output frequency, as a quantity in Hz.
		"""

		return float(self.ask('source:frequency:cw?'))

	@frequency.setter
	@quantity_unwrapped('Hz')
	def frequency(self, value):
		if value < self.min_freq or value > self.max_freq:
			raise ValueError('Value {0} not within the allowed bounds: {1} to {2}'.format(value,
				self.min_freq, self.max_freq))

		self.write('source:frequency:cw {0}'.format(value))


name = 'SMF100A'
implementation = SMF100A



from pyparsing import (alphanums, alphas, delimitedList, nums, CaselessLiteral, Combine,
		Forward, Keyword, LineEnd, Literal, OneOrMore, Optional, ParseBaseException, ParseException,
		ParserElement, QuotedString, SkipTo, StringEnd, Suppress, Word, ZeroOrMore)

from ..units import Quantity
from .tool.box import find_location, format_error
from .tree import (Acquire, Assignment, Attribute, Block, Declaration, Delay, Dictionary,
		DictionaryItem, Loop, ParallelPulses, Pulse, PulseSequence, Variable)

"""
A parser for pulse programs.
"""


class PulseError(Exception):
	"""
	A problem with the pulse program.
	"""

	pass

class PulseSyntaxError(PulseError):
	"""
	Could not parse pulse program.
	"""

	pass


def read_quantity(s, loc, toks):
	"""
	Attempt to create a Quantity object.
	"""

	try:
		return Quantity(toks[0], toks[1])
	except ValueError as e:
		raise ParseException(s, loc, e)


def Parser(raw=False):
	"""
	Create a pulse program parser.

	If raw is True, returns the pyparsing parser object.
	Otherwise, returns a function which takes a string and returns an AST.
	"""

	old_whitespace = ParserElement.DEFAULT_WHITE_CHARS

	try:
		# Handle line breaks manually.
		ParserElement.setDefaultWhitespaceChars(ParserElement.DEFAULT_WHITE_CHARS.replace('\n', ''))

		# Keywords.
		## Types.
		DELAY, INT, OUTPUT, PULSE = Keyword('delay'), Keyword('int'), Keyword('output'), Keyword('pulse')

		## Syntax.
		ACQUIRE = Keyword('acquire').suppress().setName('acquire')
		TERMINATOR = (LineEnd() | ';').suppress().setName('terminator')
		TIMES = Keyword('times').suppress().setName('times')

		# Values.
		identifier = Word(alphas + '_', alphanums + '_')
		identifier.setName('identifier')

		## Numbers.
		### Integer.
		unparsed_inum = Word('+-' + nums, nums)
		inum = unparsed_inum.copy().setParseAction(lambda x: int(x[0]))
		inum.setName('inum')

		### Floating point.
		dot = Literal('.')
		e = CaselessLiteral('e')

		fractional_part = dot + Word(nums, nums)
		exponent_part = e + inum

		fnum = Combine(unparsed_inum + (fractional_part + Optional(exponent_part) | exponent_part))
		fnum.setParseAction(lambda x: float(x[0]))
		fnum.setName('fnum')

		number = fnum | inum

		## Quantities.
		# Unit symbol cannot start with "e".
		unit_symbol = Combine(Word(alphas.replace('E', '').replace('e', ''), alphas) + Optional(Word(nums)))
		unit_symbols = delimitedList(unit_symbol, delim='.', combine=True)
		quantity = (number + unit_symbols).setParseAction(read_quantity)
		quantity.setName('quantity')

		## Strings.
		string = QuotedString(r'"', escChar=r'\\') | QuotedString(r"'", escChar=r'\\')

		value = quantity | number | string

		## Dictionaries.
		dictionary_item = (identifier('key') + Suppress(':') + value('value')).setParseAction(DictionaryItem)
		dictionary = Suppress('{') + Optional(delimitedList(dictionary_item)) + Suppress('}')
		dictionary.setParseAction(Dictionary)

		# Variables.
		type = DELAY | INT | OUTPUT | PULSE

		attribute = (identifier('variable') + Suppress('.') - identifier('name')).setParseAction(Attribute)

		identifier_assignment = identifier('target') + Suppress('=') - (dictionary | value)('value')
		identifier_assignment.setName('identifier_assignment')

		attribute_assignment = attribute('target') + Suppress('=') - (value)('value')
		attribute_assignment.setName('attribute_assignment')

		assignment = (identifier_assignment | attribute_assignment).setParseAction(Assignment)

		declaration_list = delimitedList(assignment | identifier('name').setParseAction(Variable))
		declaration = (type('type') + declaration_list('variables')).setParseAction(Declaration)

		# Commands.
		## Acquire.
		acquire = ACQUIRE.setParseAction(Acquire)

		## Delay.
		delay = (quantity | identifier)('length').setParseAction(Delay)

		## Pulse.
		pulse_sequence = (Suppress('(') + OneOrMore(identifier | delay) + Suppress(')')) | identifier
		pulse_sequence.setParseAction(PulseSequence)
		pulse = (pulse_sequence('sequence') + Suppress(':') + identifier('target')).setParseAction(Pulse)
		pulses = OneOrMore(pulse).setParseAction(ParallelPulses)

		command = acquire | pulses | delay

		# Blocks.
		block = Forward().setParseAction(Block)
		loop_block = (TIMES + (identifier | inum)('times') + block('block')).setParseAction(Loop)

		# Statements.
		statement = Optional(assignment | declaration | command)
		# The last statement does not require a terminator.
		statements = ZeroOrMore(loop_block | statement + TERMINATOR) + statement

		block << (Suppress('{') + statements + Suppress('}'))

		parser = (statements + StringEnd().suppress()).setParseAction(Block)

		# Comments.
		comment = Literal('#') + SkipTo(LineEnd())
		parser.ignore(comment)

		if raw:
			return parser
		else:
			def parseString(s):
				s = s.expandtabs()

				try:
					return parser.parseString(s)[0]
				except ParseBaseException as e:
					raise PulseSyntaxError([format_error(e.msg, *find_location(s, e.loc))])

			return parseString
	finally:
		ParserElement.setDefaultWhitespaceChars(old_whitespace)



import logging
log = logging.getLogger(__name__)

import struct

from spacq.interface.resources import Resource
from spacq.interface.units import Quantity
from spacq.tool.box import Synchronized

from ..abstract_device import AbstractDevice, AbstractSubdevice
from ..tools import str_to_bool, quantity_wrapped, quantity_unwrapped, BlockData

"""
Tektronix AWG5014B Arbitrary Waveform Generator

Control the AWG's settings and output waveforms.
"""


class Marker(AbstractSubdevice):
	"""
	Marker channel of an output channel.
	"""

	def _setup(self):
		AbstractSubdevice._setup(self)

		# Resources.
		read_write = ['delay', 'high', 'low']
		for name in read_write:
			self.resources[name] = Resource(self, name, name)

		self.resources['delay'].units = 's'
		self.resources['high'].units = 'V'
		self.resources['low'].units = 'V'

	def __init__(self, device, channel, number, *args, **kwargs):
		AbstractSubdevice.__init__(self, device, *args, **kwargs)

		self.channel = channel
		self.number = number

	@property
	@quantity_wrapped('s')
	def delay(self):
		"""
		The marker delay, as a quantity in s.
		"""

		return float(self.device.ask('source{0}:marker{1}:delay?'.format(self.channel, self.number)))

	@delay.setter
	@quantity_unwrapped('s')
	def delay(self, v):
		self.device.write('source{0}:marker{1}:delay {2}'.format(self.channel, self.number, v))

	@property
	@quantity_wrapped('V')
	def high(self):
		"""
		The marker high voltage, as a quantity in V.
		"""

		return float(self.device.ask('source{0}:marker{1}:voltage:high?'.format(self.channel, self.number)))

	@high.setter
	@quantity_unwrapped('V')
	def high(self, v):
		self.device.write('source{0}:marker{1}:voltage:high {2}'.format(self.channel, self.number, v))

	@property
	@quantity_wrapped('V')
	def low(self):
		"""
		The marker low voltage, as a quantity in V.
		"""

		return float(self.device.ask('source{0}:marker{1}:voltage:low?'.format(self.channel, self.number)))

	@low.setter
	@quantity_unwrapped('V')
	def low(self, v):
		self.device.write('source{0}:marker{1}:voltage:low {2}'.format(self.channel, self.number, v))


class Channel(AbstractSubdevice):
	"""
	Output channel of the AWG.
	"""

	# Zero-to-peak amplitude range.
	min_amplitude = 0.01 # V
	max_amplitude = 2.25 # V

	def _setup(self):
		AbstractSubdevice._setup(self)

		self.markers = [None] # There is no marker 0.
		for mark in xrange(1, 3):
			marker = Marker(self.device, self.channel, mark)
			self.markers.append(marker)
			self.subdevices['marker{0}'.format(mark)] = marker

		# Resources.
		read_write = ['waveform_name', 'enabled', 'amplitude']
		for name in read_write:
			self.resources[name] = Resource(self, name, name)

		self.resources['enabled'].converter = str_to_bool
		self.resources['amplitude'].units = 'V'

	def __init__(self, device, channel, *args, **kwargs):
		self.channel = channel

		AbstractSubdevice.__init__(self, device, *args, **kwargs)

	@property
	def waveform_name(self):
		"""
		The name of the output waveform for the channel.
		"""

		result = self.device.ask('source{0}:waveform?'.format(self.channel))
		# The name is in quotes.
		return result[1:-1]

	@waveform_name.setter
	def waveform_name(self, v):
		self.device.write('source{0}:waveform "{1}"'.format(self.channel, v))

	@waveform_name.deleter
	def waveform_name(self):
		self.waveform_name = ''

	@property
	def enabled(self):
		"""
		The output state (on/off) of the channel.
		"""

		result = self.device.ask('output{0}:state?'.format(self.channel))
		# The value is a string holding a digit.
		return bool(int(result))

	@enabled.setter
	def enabled(self, v):
		self.device.write('output{0}:state {1}'.format(self.channel, int(v)))

	@property
	@quantity_wrapped('V')
	def amplitude(self):
		"""
		The zero-to-peak amplitude of the channel in V.
		"""

		# Convert peak-to-peak to zero-to-peak.
		return float(self.device.ask('source{0}:voltage?'.format(self.channel))) / 2

	@amplitude.setter
	@quantity_unwrapped('V')
	def amplitude(self, v):
		# Convert zero-to-peak to peak-to-peak.
		self.device.write('source{0}:voltage {1:E}'.format(self.channel, 2 * v))

	def set_waveform(self, waveform, markers=None, name=None):
		"""
		Set the waveform on this channel.

		The waveform data should be in V.
		"""

		if name is None:
			name = 'Channel {0}'.format(self.channel)

		# Clear existing.
		if name in self.device.waveform_names:
			self.device.delete_waveform(name)

		# Normalize waveform.
		max_amp = max(abs(x) for x in waveform)
		if max_amp > self.max_amplitude:
			raise ValueError('Amplitude {0} V exceeds maximum of {1} V'.format(max_amp, self.max_amplitude))
		elif max_amp > 0:
			if max_amp < self.min_amplitude:
				max_amp = self.min_amplitude

			waveform = [x / max_amp for x in waveform]

			self.amplitude = Quantity(max_amp, 'V')

		# Create new.
		self.device.create_waveform(name, waveform, markers)
		self.waveform_name = name


class AWG5014B(AbstractDevice):
	"""
	Interface for Tektronix AWG5014B AWG.
	"""

	allowed_run_modes = set(['continuous', 'triggered', 'gated', 'sequence'])

	def _setup(self):
		AbstractDevice._setup(self)

		self.channels = [None] # There is no channel 0.
		for chan in xrange(1, 5):
			channel = Channel(self, chan)
			self.channels.append(channel)
			self.subdevices['channel{0}'.format(chan)] = channel

		# Resources.
		read_write = ['sampling_rate', 'run_mode', 'enabled']
		for name in read_write:
			self.resources[name] = Resource(self, name, name)

		self.resources['sampling_rate'].units = 'Hz'
		self.resources['run_mode'].allowed_values = self.allowed_run_modes
		self.resources['enabled'].converter = str_to_bool

	@Synchronized()
	def reset(self):
		"""
		Reset the device to its default state.
		"""

		log.info('Resetting "{0}".'.format(self.name))
		self.write('*rst')

	@property
	def data_bits(self):
		"""
		How many bits of each data point represent the data itself.
		"""

		return 14

	@property
	def value_range(self):
		"""
		The range of values possible for each data point.
		"""

		# The sent values are unsigned.
		return (0, 2 ** self.data_bits - 1)

	@property
	@quantity_wrapped('Hz')
	def sampling_rate(self):
		"""
		The sampling rate of the AWG in Hz.
		"""

		return float(self.ask('source1:frequency?'))

	@sampling_rate.setter
	@quantity_unwrapped('Hz')
	def sampling_rate(self, value):
		if value < 1e7 or value > 1.2e9:
			raise ValueError('Sampling rate must be between 1e7 Hz and 1.2e9 Hz, not {0:n} Hz'.format(value))

		self.write('source1:frequency {0:E}'.format(value))

	@property
	def run_mode(self):
		"""
		The run mode of the AWG. One of: continuous, triggered, gated, sequence.
		"""

		mode = self.ask('awgcontrol:rmode?').lower()

		if mode.startswith('cont'):
			return 'continuous'
		elif mode.startswith('trig'):
			return 'triggered'
		elif mode.startswith('gat'):
			return 'gated'
		elif mode.startswith('seq'):
			return 'sequence'
		else:
			ValueError('Unknown mode: {0}'.format(mode))

	@run_mode.setter
	def run_mode(self, value):
		if value not in self.allowed_run_modes:
			raise ValueError('Invalid run mode: {0}'.format(value))

		self.write('awgcontrol:rmode {0}'.format(value))

	@property
	@Synchronized()
	def waveform_names(self):
		"""
		A list of all waveforms in the AWG.
		"""

		num_waveforms = int(self.ask('wlist:size?'))

		self.multi_command_start()
		# Waveforms on the AWG are numbered from 0.
		for i in xrange(num_waveforms):
			self.ask('wlist:name? {0}'.format(i))
		names = self.multi_command_stop()

		# Names are in quotes.
		return [name[1:-1] for name in names]

	@Synchronized()
	def get_waveform(self, name):
		self.status.append('Getting waveform "{0}"'.format(name))

		try:
			log.debug('Getting waveform "{0}" from device "{1}".'.format(name, self.name))

			block_data = self.ask_raw('wlist:waveform:data? "{0}"'.format(name))
			packed_data = BlockData.from_block_data(block_data)
			waveform_length = len(packed_data) / 2
			data = struct.unpack('<{0}H'.format(waveform_length), packed_data)
			data = [x & 2 ** 14 - 1 for x in data] # Filter out marker data.

			min_value, max_value = self.value_range
			range_diff = max_value - min_value
			data = [2.0 * (x - min_value) / range_diff - 1.0 for x in data]

			log.debug('Got waveform "{0}" from device "{1}": {2!r}'.format(name, self.name, data))

			return data
		finally:
			self.status.pop()

	@Synchronized()
	def create_waveform(self, name, data, markers=None):
		"""
		Create a new waveform on the AWG.

		The waveform data should be on [-1, 1].
		"""

		self.status.append('Creating waveform "{0}"'.format(name))

		try:
			log.debug('Creating waveform "{0}" on device "{1}" with data: {2!r}'.format(name, self.name, data))

			min_value, max_value = self.value_range
			range_diff = max_value - min_value
			data = [min_value + int(range_diff * (x + 1.0) / 2.0) for x in data]

			waveform_length = len(data)
			self.write('wlist:waveform:new "{0}", {1}, integer'.format(name, waveform_length))

			if markers:
				# The markers are in the top 2 bits.
				for marker_num, marker_bit in zip([1, 2], [1 << 14, 1 << 15]):
					try:
						for i, marker_datum in enumerate(markers[marker_num]):
							if marker_datum:
								data[i] += marker_bit
						log.debug('Added marker {0} to waveform "{1}" device "{1}": {2!r}'.format(marker_num,
								name, self.name, markers[marker_num]))
					except KeyError:
						pass

				extra_markers = set(markers) - set([1, 2])
				for extra in extra_markers:
					log.warning('Marker {0} ignored: {1!r}'.format(extra, markers[extra]))

			# Always 16-bit, unsigned, little-endian.
			packed_data = struct.pack('<{0}H'.format(waveform_length), *data)
			block_data = BlockData.to_block_data(packed_data)

			log.debug('Sending packed block waveform data for "{0}" on device "{1!r}": {2}'.format(name,
					self.name, block_data))

			self.write('wlist:waveform:data "{0}", {1}'.format(name, block_data))
		finally:
			self.status.pop()

	def delete_waveform(self, name):
		"""
		Remove a waveform on the AWG.
		"""

		if name not in self.waveform_names:
			raise ValueError('No such waveform "{0}"'.format(name))

		self.write('wlist:waveform:delete "{0}"'.format(name))

	@property
	def enabled(self):
		"""
		The continuous run state (on/off) of the AWG.
		"""

		state = self.ask('awgcontrol:rstate?')

		if state == '0':
			return False
		elif state in ['1', '2']:
			# Either on or waiting for trigger.
			return True
		else:
			raise ValueError('State "{0}" not implemented.'.format(state))

	@enabled.setter
	def enabled(self, v):
		if v:
			log.debug('Enabling "{0}".'.format(self.name))

			self.write('awgcontrol:run')
		else:
			log.debug('Disabling "{0}".'.format(self.name))

			self.write('awgcontrol:stop')

	@property
	def waiting_for_trigger(self):
		"""
		Whether the AWG is waiting for a trigger.
		"""

		return '1' == self.ask('awgcontrol:rstate?')

	def trigger(self):
		"""
		Force a trigger event.
		"""

		self.write('*trg')

	def clear_channels(self):
		"""
		Delete all waveforms from all channels.
		"""

		for channel in self.channels[1:]:
			del channel.waveform_name


name = 'AWG5014B'
implementation = AWG5014B



from os import path
from nose.tools import assert_raises, eq_
from numpy.testing import assert_array_almost_equal, assert_array_equal
from unittest import main, TestCase

from ...units import Quantity
from ..parser import PulseSyntaxError

from .. import program


resource_dir = path.join(path.dirname(__file__), 'resources')


class ProgramTest(TestCase):
	missing = [
		(('_acq_marker', 'marker_num'), 1),
		(('_acq_marker', 'output'), 'f1'),
		(('first_square', 'amplitude'), Quantity(0.5, 'V')),
		(('last_square', 'amplitude'), Quantity(-0.5, 'V')),
		(('last_square', 'length'), Quantity(5, 'ns')),
		(('manipulator', 'amplitude'), Quantity(1, 'V')),
		(('manipulator', 'length'), Quantity(12, 'ns')),
		(('settle',), Quantity(20, 'ns')),
	]

	def testFromFile(self):
		"""
		Grab a file and load the program in it.
		"""

		p = program.Program.from_file(path.join(resource_dir, '01.pulse'))

		p.set_value(('wobble', 'shape'), 'non-square')

		eq_(p.missing_values, set([name for name, value in self.missing]))
		eq_(set(p.variables), set(['bumps', 'bump_spacing', 'settle', 'end_delay', 'first_square', 'wobble', 'last_square', 'manipulator', 'f1', 'f2', '_acq_marker']))

	def testInvalid(self):
		"""
		This program isn't syntactically correct.
		"""

		assert_raises(PulseSyntaxError, program.Program.from_file, path.join(resource_dir, '02.pulse'))

	def testInvalidShapePath(self):
		"""
		A shape file which isn't there.
		"""

		p = program.Program.from_file(path.join(resource_dir, '01.pulse'))

		for name, value in self.missing:
			p.set_value(name, value)

		p.set_value(('wobble', 'shape'), 'this-shape-doesn\'t-exist')

		try:
			p.generate_waveforms()
		except program.PulseError as e:
			eq_('\n'.join(e[0]), """\
error: File "this-shape-doesn't-exist" (due to "wobble") not found at column 17 on line 21:
  first_square:f1 wobble:f2
                  ^\
""")
		else:
			assert False, 'Expected PulseError'

	def testInvalidShapeFile(self):
		"""
		A shape file which isn't a shape file.
		"""

		p = program.Program.from_file(path.join(resource_dir, '01.pulse'))

		for name, value in self.missing:
			p.set_value(name, value)

		assert ('wobble', 'shape') not in p.values
		p.set_value(('wobble', 'shape'), '01.pulse')
		assert ('wobble', 'shape') in p.values

		assert_raises(ValueError, p.generate_waveforms)

	def testGenerateWaveforms(self):
		"""
		Finally generate some waveforms.
		"""

		p = program.Program.from_file(path.join(resource_dir, '01.pulse'))

		# Missing values.
		assert_raises(ValueError, p.generate_waveforms)

		for name, value in self.missing:
			p.set_value(name, value)

		p.set_value(('wobble', 'shape'), 'non-square')

		p.frequency = Quantity(1, 'GHz')
		eq_(p.frequency, Quantity(1, 'GHz'))
		waveforms = p.generate_waveforms()

		eq_(set(waveforms.keys()), set(['f1', 'f2']))

		f1 = waveforms['f1']
		loop = [0.0] * 10 + [0.5] + [0.0] * 7 + [0.5] + [0.0] * 7
		assert_array_equal(f1.data, [0.0] * 10 + [0.5] * 1 + [0.0] * 7 + loop * 2 + [0.0] * 20 + [-0.5] * 5 + [0.0] * 49)
		eq_(f1.markers[1], [False] * 90 + [True] * 54)

		f2 = waveforms['f2']
		f2_gen = p._env.generators['f2']
		non_square = [0.1, 0.5, 0.7, 1.0] + [4.2] * 3 + [3.6, 9.9]
		wobble = f2_gen._scale_waveform(non_square, Quantity(-1, 'mV').value, Quantity(8, 'ns'))
		manipulator = f2_gen._scale_waveform(non_square, Quantity(1, 'V').value, Quantity(12, 'ns'))
		loop = [0.0] * 10 + wobble * 2
		end = manipulator + [0.0] * 15
		assert_array_almost_equal(f2.data, [0.0] * 10 + wobble + loop * 2 + [0.0] * 20 + end * 2, 2)

	def testWaveformsDryRun(self):
		"""
		Run through waveform generation, but don't actually generate anything.
		"""

		p = program.Program.from_file(path.join(resource_dir, '01.pulse'))

		# Missing values.
		assert_raises(ValueError, p.generate_waveforms, dry_run=True)

		for name, value in self.missing:
			p.set_value(name, value)

		p.set_value(('wobble', 'shape'), 'non-square')

		# Too long.
		p.frequency = Quantity(1, 'YHz')
		assert_raises(ValueError, p.generate_waveforms, dry_run=True)

		# Just right.
		p.frequency = Quantity(1, 'GHz')
		waveforms = p.generate_waveforms(dry_run=True)

		# But nothing there.
		eq_(set(waveforms.keys()), set(['f1', 'f2']))
		eq_(list(waveforms['f1'].data), [])
		eq_(waveforms['f1'].markers, {})
		eq_(list(waveforms['f2'].data), [])
		eq_(waveforms['f2'].markers, {})


if __name__ == '__main__':
	main()



from nose.tools import assert_raises, eq_
from numpy.testing import assert_array_almost_equal
from unittest import main, TestCase

from ..units import Quantity

from .. import waveform


class GeneratorTest(TestCase):
	def testEmpty(self):
		"""
		Generate an empty waveform.
		"""

		wg = waveform.Generator(frequency=Quantity(1, 'Hz'))

		eq_(list(wg.waveform.data), [])
		eq_(wg.waveform.markers, {})

	def testWaveform(self):
		"""
		Generate a non-empty waveform.
		"""

		wg = waveform.Generator(frequency=Quantity(2, 'Hz'))

		wg.delay(Quantity(2, 's'))
		wg.marker(1, True)
		wg.marker(2, True)
		wg.pulse([], 0.5, Quantity(1, 's'))
		wg.pulse([1.0, 0.0, -1.0], 1.0, Quantity(3, 's'))
		wg.marker(1, False)
		wg.square(-0.5, Quantity(2, 's'))

		expected = [0.0, 0.0, 0.0, 1.0, 0.6, 0.2, -0.2, -0.6, -1.0, -0.5, -0.5, -0.5, -0.5, -1.0]

		wave, markers = wg.waveform
		assert_array_almost_equal(wave, expected, 4)
		eq_(markers[1], [False] * 3 + [True] * 6 + [False] * 5)
		eq_(markers[2], [False] * 3 + [True] * 11)
		assert 3 not in markers

	def testEndWithMarker(self):
		"""
		Marker data is longer than waveform data.
		"""

		wg = waveform.Generator(frequency=Quantity(1, 'mHz'))

		wg.marker(1, True)
		wg.marker(2, False)

		wave, markers = wg.waveform
		eq_(wave, [0.0])
		eq_(markers, {1: [True], 2: [False]})

	def testTooLong(self, dry_run=False):
		"""
		Try to create a waveform that is far too long.
		"""

		wg = waveform.Generator(frequency=Quantity(1, 'GHz'), dry_run=dry_run)

		wg.delay(Quantity(1, 'ns'))
		wg.delay(Quantity(1, 'us'))
		wg.delay(Quantity(1, 'ms'))
		assert_raises(ValueError, wg.delay, Quantity(0.01, 's'))

	def testDryRun(self):
		"""
		testTooLong, but as a dry run.
		"""

		self.testTooLong(dry_run=True)


if __name__ == '__main__':
	main()



from functools import partial
from nose.tools import eq_
from os import path
from threading import Thread
from time import sleep, time
from unittest import main, TestCase

from spacq.devices.config import DeviceConfig
from spacq.interface.pulse.program import Program
from spacq.interface.resources import Resource
from spacq.interface.units import Quantity
from spacq.tool.box import flatten

from ..variables import sort_variables, InputVariable, OutputVariable, LinSpaceConfig

from .. import sweep


resource_dir = path.join(path.dirname(__file__), 'resources')


class SweepControllerTest(TestCase):
	def testSingle(self):
		"""
		Iterate over a single thing without any measurements.
		"""

		res_buf = []

		def setter(value):
			res_buf.append(value)

		res = Resource(setter=setter)
		var = OutputVariable(name='Var', order=1, enabled=True, const=-1.0)
		var.config = LinSpaceConfig(1.0, 4.0, 4)
		var.smooth_steps = 3
		var.smooth_from, var.smooth_to = [True] * 2

		vars, num_items = sort_variables([var])
		ctrl = sweep.SweepController([(('Res', res),)], vars, num_items, [], [])

		# Callback verification buffers.
		actual_values = []
		actual_measurement_values = []
		actual_writes = []
		actual_reads = []
		closed = [0]

		# Callbacks.
		def data_callback(cur_time, values, measurement_values):
			actual_values.append(values)
			actual_measurement_values.append(measurement_values)
		ctrl.data_callback = data_callback

		def close_callback():
			closed[0] += 1
		ctrl.close_callback = close_callback

		def write_callback(pos, i, value):
			actual_writes.append((pos, i, value))
		ctrl.write_callback = write_callback

		def read_callback(i, value):
			actual_reads.append((i, value))
		ctrl.read_callback = read_callback

		# Let it run.
		ctrl.run()

		eq_(res_buf, [-1.0, 0.0, 1.0, 1.0, 2.0, 3.0, 4.0, 4.0, 1.5, -1.0])
		eq_(actual_values, [(1.0,), (2.0,), (3.0,), (4.0,)])
		eq_(actual_measurement_values, [()] * 4)
		eq_(actual_writes, [(0, 0, x) for x in [1.0, 2.0, 3.0, 4.0]])
		eq_(actual_reads, [])
		eq_(closed, [1])

	def testProper(self):
		"""
		Testing everything that there is to test along the happy path:
			nested and parallel variables
			measurements
			dwell time
		"""

		res_bufs = [[], [], [], []]
		measurement_counts = [0] * 2

		def setter(i, value):
			res_bufs[i].append(value)

		def getter(i):
			measurement_counts[i] += (-1) ** i

			return measurement_counts[i]

		dwell_time = Quantity(50, 'ms')

		# Output.
		res0 = Resource(setter=partial(setter, 0))
		res0.units = 'cm-1'
		res1 = Resource(setter=partial(setter, 1))
		res2 = Resource(setter=partial(setter, 2))
		res3 = Resource(setter=partial(setter, 3))

		var0 = OutputVariable(name='Var 0', order=2, enabled=True, const=0.0)
		var0.config = LinSpaceConfig(-1.0, -2.0, 2)
		var0.smooth_steps = 2
		var0.smooth_from, var0.smooth_to, var0.smooth_transition = [True] * 3
		var0.type = 'quantity'
		var0.units = 'cm-1'

		var1 = OutputVariable(name='Var 1', order=1, enabled=True, const=-1.0)
		var1.config = LinSpaceConfig(1.0, 4.0, 4)
		var1.smooth_steps = 3
		var1.smooth_from, var1.smooth_to, var1.smooth_transition = [True] * 3

		var2 = OutputVariable(name='Var 2', order=1, enabled=True, const=1.23, use_const=True)

		var3 = OutputVariable(name='Var 3', order=1, enabled=True, const=-9.0, wait=str(dwell_time))
		var3.config = LinSpaceConfig(-1.0, 2.0, 4)
		var3.smooth_steps = 2
		var3.smooth_from, var3.smooth_to, var3.smooth_transition = True, True, False

		var4 = OutputVariable(name='Var 4', order=3, enabled=True, const=-20.0)
		var4.config = LinSpaceConfig(-10.0, 20, 1)
		var4.smooth_steps = 2
		var4.smooth_from = True

		# Input.
		meas_res0 = Resource(getter=partial(getter, 0))
		meas_res1 = Resource(getter=partial(getter, 1))

		meas0 = InputVariable(name='Meas 0')
		meas1 = InputVariable(name='Meas 1')

		vars, num_items = sort_variables([var0, var1, var2, var3, var4])
		ctrl = sweep.SweepController([(('Res 2', res2),), (('Something', None),), (('Res 0', res0),),
				(('Res 1', res1), ('Res 3', res3))], vars, num_items,
				[('Meas res 0', meas_res0), ('Meas res 1', meas_res1)], [meas0, meas1])

		# Callback verification buffers.
		actual_values = []
		actual_measurement_values = []
		actual_writes = []
		actual_reads = []
		closed = [0]

		# Callbacks.
		def data_callback(cur_time, values, measurement_values):
			actual_values.append(values)
			actual_measurement_values.append(measurement_values)
		ctrl.data_callback = data_callback

		def close_callback():
			closed[0] += 1
		ctrl.close_callback = close_callback

		def write_callback(pos, i, value):
			actual_writes.append((pos, i, value))
		ctrl.write_callback = write_callback

		def read_callback(i, value):
			actual_reads.append((i, value))
		ctrl.read_callback = read_callback

		# Let it run.
		start_time = time()
		ctrl.run()
		elapsed_time = time() - start_time

		expected_time = num_items * dwell_time.value
		assert expected_time < elapsed_time, 'Took {0} s, expected at least {1} s.'.format(elapsed_time, expected_time)

		expected_res1 = [1.0, 2.0, 3.0, 4.0]
		expected_res2 = [-1.0, 0.0, 1.0, 2.0]

		expected_inner_writes = list(flatten(((3, 0, x), (3, 1, x - 2.0)) for x in [1.0, 2.0, 3.0, 4.0]))
		expected_writes = [(0, 0, 1.23), (1, 0, -10.0)] + list(flatten([(2, 0, x)] + expected_inner_writes
				for x in [Quantity(x, 'cm-1') for x in [-1.0, -2.0]]))

		eq_(res_bufs, [
			[Quantity(x, 'cm-1') for x in [0.0, -1.0, -1.0, -2.0, -2.0, 0.0]],
			[-1.0, 0.0, 1.0] + expected_res1 + [4.0, 2.5, 1.0] + expected_res1 + [4.0, 1.5, -1.0],
			[1.23],
			[-9.0, -1.0] + expected_res2 + expected_res2 + [2.0, -9.0],
		])
		eq_(measurement_counts, [8, -8])
		eq_(actual_values, [(1.23, -10.0, x, y, y - 2.0)
				for x in [Quantity(x, 'cm-1') for x in [-1.0, -2.0]]
				for y in [1.0, 2.0, 3.0, 4.0]])
		eq_(actual_measurement_values, [(x, -x) for x in xrange(1, 9)])
		eq_(actual_writes, expected_writes)
		eq_(actual_reads, list(flatten(((0, x), (1, -x)) for x in xrange(1, 9))))
		eq_(closed, [1])

	def testContinuous(self):
		"""
		Keep going, and then eventually stop.
		"""

		res_buf = []

		def setter(value):
			res_buf.append(value)

		res = Resource(setter=setter)
		var = OutputVariable(name='Var', order=1, enabled=True)
		var.config = LinSpaceConfig(1.0, 4.0, 4)

		vars, num_items = sort_variables([var])
		ctrl = sweep.SweepController([(('Res', res),)], vars, num_items, [], [], continuous=True)

		thr = Thread(target=ctrl.run)
		thr.daemon = True
		thr.start()

		sleep(0.5)
		ctrl.pause()
		sleep(0.5)
		ctrl.unpause()
		sleep(0.5)

		ctrl.last_continuous = True
		thr.join()

		expected_buf = [1.0, 2.0, 3.0, 4.0]

		eq_(res_buf[:len(expected_buf) * 50], expected_buf * 50)

	def testWriteException(self):
		"""
		Fail to read.
		"""

		exceptions = []
		e = ValueError()

		def setter(value):
			raise e

		res = Resource(setter=setter)
		var = OutputVariable(name='Var', order=1, enabled=True, const=0.0)
		var.config = LinSpaceConfig(1.0, 4.0, 4)

		vars, num_items = sort_variables([var])
		ctrl = sweep.SweepController([(('Res', res),)], vars, num_items, [], [])

		def resource_exception_handler(name, e, write):
			exceptions.append((name, e))
			ctrl.abort(fatal=True)
			assert write
		ctrl.resource_exception_handler = resource_exception_handler

		ctrl.run()

		eq_(exceptions, [('Res', e)])

	def testReadException(self):
		"""
		Fail to write.
		"""

		exceptions = []
		e = ValueError()

		def getter():
			raise e

		res = Resource(setter=lambda x: x)
		var = OutputVariable(name='Var', order=1, enabled=True)
		var.config = LinSpaceConfig(1.0, 4.0, 4)

		meas_res = Resource(getter=getter)
		meas_var = InputVariable(name='Meas var')

		vars, num_items = sort_variables([var])
		ctrl = sweep.SweepController([(('Res', res),)], vars, num_items, [('Meas res', meas_res)], [meas_var])

		def resource_exception_handler(name, e, write):
			exceptions.append((name, e))
			assert not write
		ctrl.resource_exception_handler = resource_exception_handler

		ctrl.run()

		eq_(exceptions, [('Meas res', e)] * 4)

	def testPulseProgram(self):
		"""
		Iterate with a pulse program.
		"""

		res_buf = []

		def setter(value):
			res_buf.append(value)

		res = Resource(setter=setter)
		var1 = OutputVariable(name='Var 1', order=1, enabled=True)
		var1.config = LinSpaceConfig(1.0, 4.0, 4)

		p = Program.from_file(path.join(resource_dir, '01.pulse'))
		p.frequency = Quantity(1, 'GHz')
		p.set_value(('_acq_marker', 'marker_num'), 1)
		p.set_value(('_acq_marker', 'output'), 'f1')

		eq_(p.all_values, set([('_acq_marker', 'marker_num'), ('_acq_marker', 'output'), ('d',), ('i',),
				('p', 'amplitude'), ('p', 'length'), ('p', 'shape')]))

		parameters = [('i',), ('d',), ('p', 'amplitude'), ('p', 'length')]
		for parameter in parameters:
			p.resource_labels[parameter] = 'res_' + '.'.join(parameter)
			p.resources[parameter] = Resource()

		var2 = OutputVariable(name='Var 2', order=1, enabled=True)
		var2.config = LinSpaceConfig(1, 4, 4)
		var2.type = 'integer'

		awg_cfg = DeviceConfig('awg')
		awg_cfg.address_mode = awg_cfg.address_modes.gpib
		awg_cfg.manufacturer, awg_cfg.model = 'Tektronix', 'AWG5014B'
		awg_cfg.mock = True
		awg_cfg.connect()

		osc_cfg = DeviceConfig('osc')
		osc_cfg.address_mode = awg_cfg.address_modes.gpib
		osc_cfg.manufacturer, osc_cfg.model = 'Tektronix', 'DPO7104'
		osc_cfg.mock = True
		osc_cfg.connect()

		pulse_config = sweep.PulseConfiguration(p.with_resources, {'f1': 1}, awg_cfg.device, osc_cfg.device)

		vars, num_items = sort_variables([var1, var2])
		ress = [(('Res 1', res), ('Res 2', p.resources[('i',)]))]
		ctrl = sweep.SweepController(ress, vars, num_items, [], [], pulse_config)

		ctrl.run()

		eq_(res_buf, [1.0, 2.0, 3.0, 4.0])


if __name__ == '__main__':
	main()



from chaco.api import Plot
from chaco.tools.api import BetterSelectingZoom, PanTool

"""
Chaco wrapper.
"""


class ChacoPlot(Plot):
	"""
	A 2D Chaco plot wrapped with useful common functionality.
	"""

	@staticmethod
	def sci_formatter(value):
		"""
		Convert a value to a scientific notation string as applicable.
		"""

		# Subtly different from g or n presentation types.
		if value != 0 and (abs(value) < 1e-3 or abs(value) > 1e3):
			parts = '{0:e}'.format(value).split('e')
			result = parts[0].rstrip('0').rstrip('.') + 'e' + parts[1]
		else:
			result = '{0:f}'.format(value).rstrip('0').rstrip('.')

		return result

	def configure(self):
		"""
		Configure padding, tools, etc.
		"""

		# Padding.
		self.padding = 20
		self.padding_left = 120
		self.padding_bottom = 55

		# Axes.
		self.index_axis.tick_label_formatter = self.sci_formatter
		self.value_axis.tick_label_formatter = self.sci_formatter

		# Tools.
		self.tools.append(PanTool(self))

		zoom = BetterSelectingZoom(self)
		self.tools.append(zoom)
		self.overlays.append(zoom)

	@property
	def x_label(self):
		"""
		The x axis label.
		"""

		return self.index_axis.title

	@x_label.setter
	def x_label(self, value):
		self.index_axis.title = value

	@property
	def y_label(self):
		"""
		The y axis label.
		"""

		return self.value_axis.title

	@y_label.setter
	def y_label(self, value):
		self.value_axis.title = value



from nose.plugins.skip import SkipTest
from nose.tools import assert_raises, eq_
from unittest import main, TestCase

from testconfig import config as tc

from ... import abstract_device


class AbstractDeviceTest(TestCase):
	def obtain_device(self, device):
		"""
		If the given device description is an Ethernet or GPIB device, return a connected device object.
		"""

		if ('address' not in device and 'ip_address' not in device['address'] and
				'gpib_pad' not in device['address']):
			return None

		try:
			return abstract_device.AbstractDevice(**device['address'])
		except Exception:
			return None

	def testAskRaw(self):
		"""
		Converse briefly with real devices.
		"""

		found_any = False

		# Try all devices to which a connection can be established.
		for name, device in tc['devices'].items():
			if not ('ip_address' in device['address'] or 'gpib_pad' in device['address']):
				continue

			dev = self.obtain_device(device)

			if dev is None:
				continue

			msg = dev.ask_raw('*idn?')
			eq_(msg[-1], '\n')

			found_any = True

		if not found_any:
			raise SkipTest('Could not connect to any device.')

	def testMultiCommand(self):
		"""
		Send a multi-command message.
		"""

		# Use any device.
		for name, device in tc['devices'].items():
			dev = self.obtain_device(device)

			if dev is None:
				continue

			# Value to check against.
			id = dev.idn

			expected = ['1'] * 3 + [id]

			assert_raises(ValueError, dev.multi_command_stop)

			# Don't actually send anything.
			dev.multi_command_start()
			responses = dev.multi_command_stop()

			eq_(responses, [])

			# Expect a response.
			dev.multi_command_start()
			dev.opc
			dev.opc
			dev.opc
			dev.idn
			responses = dev.multi_command_stop()

			eq_(responses, expected)

			return

		raise SkipTest('Could not connect to any device.')

	def testClose(self):
		"""
		Close the device.
		"""

		# Use any device.
		for name, device in tc['devices'].items():
			dev = self.obtain_device(device)

			if dev is None:
				continue

			dev.idn
			dev.opc

			dev.close()

			return

		raise SkipTest('Could not connect to any device.')


if __name__ == '__main__':
	main()



import re

from ...mock.mock_abstract_device import MockAbstractDevice
from ...tools import BinaryEncoder
from ..voltage_source import VoltageSource

"""
Mock Voltage Source

Control a fake voltage source.
"""


class MockPort(object):
	"""
	Mock port.
	"""

	def __init__(self):
		self._voltage = 0

	def set_voltage(self, voltage):
		self._voltage = voltage

	voltage = property(fset=set_voltage)


class MockVoltageSource(MockAbstractDevice, VoltageSource):
	"""
	Mock interface for Tektronix AWG5014B AWG.
	"""

	# Not sure what it means, but it comes up a lot.
	standard_reply = '0000 000c 0008 0100 0000 0002'

	def __init__(self, *args, **kwargs):
		"""
		Pretend to connect to the voltage source.
		"""

		self.port_settings = {}

		self.mocking = VoltageSource

		MockAbstractDevice.__init__(self, *args, **kwargs)

	def _reset(self):
		self.mock_state['ports'] = [MockPort() for _ in xrange(16)]

	def write(self, message, result=None, done=False):
		if done:
			MockAbstractDevice.write(self, message, result, done)

		message = BinaryEncoder.decode(message)

		# These all elicit the same response.
		uninteresting_messages = [
			'0000 0010 000c 0113 0280 0000 0000 ff01',
			'0000 0010 000c 0112 0280 0000 00ff ff00',
			'0000 0014 0010 0110 0260 0000 0000 0064 0700 0000',
		]

		if message in uninteresting_messages:
			self.output = self.standard_reply
		elif message == '0000 000c 0008 0100 0000 0000':
			self.output = '0000 001c 0018 0100 0000 0002 0200 1000 0100 c001 0100 c000 0002 000o'
		elif message.startswith('0000 0010 000c 0111 0280 0000 00ff '):
			m = re.match('0000 0010 000c 0111 0280 0000 00ff ([0-9a-f]{2})00', message)
			self.mock_state['port'] = int(m.group(1), 16)

			self.output = self.standard_reply
		elif message.startswith('0000 0014 0010 0111 0260 0000 0003 '):
			m = re.match('0000 0014 0010 0111 0260 0000 0003 ([0-9a-f]{2})00 ([0-9a-f ]{9})', message)
			len = int(m.group(1), 16)
			cmd = ''.join(['{0:08b}'.format(~ord(x) & 0xff) for x in BinaryEncoder.encode(m.group(2))[:len]])

			# Destructure the command.
			read = bool(int(cmd[0])) # Read on True, write on False.
			num_bytes = 1 + int(cmd[1:3], 2)
			register = int(cmd[4:8], 2)
			value = int(cmd[8:8+8*num_bytes], 2)

			if not read:
				if register >= 0 and register <= 2:
					# Data input register.
					value = ~value & (2 ** (8 * num_bytes) - 1)
					# Assuming here that it's OK to overwrite the lower bits with 0.
					value <<= 8 * (3 - register - num_bytes)

					self.mock_state['ports'][self.mock_state['port']].voltage = value
				elif register >= 4 and register <= 5:
					# Command register.
					# Assuming here that it's OK to overwrite the lower bits with 0.
					value <<= 8 * (6 - register - num_bytes)
					value = '{0:016b}'.format(value)

					# Destructure the CMR values.
					resolution = 20 if int(value[9]) else 16 # RES
					self.mock_state['ports'][self.mock_state['port']].resolution = resolution
					if int(value[10]): # CLR
						self.mock_state['ports'][self.mock_state['port']].voltage = 0

			# Reading never works.
			answer = 'ff' * len + '00' * (4 - len)
			self.output = '0000 0014 0010 0100 0000 0002 {0:02x}00 0000 {1}'.format(len, answer)
		else:
			self.output = None

		if self.output is not None:
			self.output = BinaryEncoder.encode(self.output)


name = 'Voltage source'
implementation = MockVoltageSource



from itertools import groupby, islice
import numpy
import operator

from spacq.interface.units import Quantity


def sort_variables(variables):
	"""
	Sort and group the variables based on their order.

	The returned values are:
		variables sorted and grouped by their order
		number of items in the Cartesian product of the orders
	"""

	# Ignore disabled variables entirely!
	variables = [var for var in variables if var.enabled]

	if not variables:
		return [], 0

	const_vars = tuple([var for var in variables if var.use_const])

	order_attr = operator.attrgetter('order')
	ordered = sorted((var for var in variables if not var.use_const), key=order_attr, reverse=True)
	grouped = [tuple(vars) for order, vars in groupby(ordered, order_attr)]

	num_items = 1
	for group in grouped:
		num_in_group = None

		for var in group:
			num_in_var = len(var)

			if num_in_group is None or num_in_var < num_in_group:
				num_in_group = num_in_var

		num_items *= num_in_group

	if const_vars:
		grouped.insert(0, const_vars)

	return grouped, num_items


class Variable(object):
	"""
	An abstract superclass for all variables.
	"""

	def __init__(self, name, enabled=False, resource_name=''):
		self.name = name
		self.enabled = enabled
		self.resource_name = resource_name


class InputVariable(Variable):
	"""
	An input (measurement) variable.
	"""

	def __init__(self, *args, **kwargs):
		Variable.__init__(self, *args, **kwargs)


class OutputVariable(Variable):
	"""
	An abstract superclass for output variables.
	"""

	# Maximum number of initial values to display in string form.
	display_values = 4

	# Maximum number of values to search through for the end.
	search_values = 1000

	def __init__(self, order, config=None, wait='0 s', const=0.0, use_const=False, *args, **kwargs):
		Variable.__init__(self, *args, **kwargs)

		self.order = order

		if config is not None:
			self.config = config
		else:
			self.config = LinSpaceConfig(0.0, 0.0, 1)

		# Iteration parameters.
		self._wait = Quantity(wait)
		self.const = const
		self.use_const = use_const

		# Smooth set.
		self.smooth_steps = 10
		self.smooth_from = False
		self.smooth_to = False
		self.smooth_transition = False

		self.type = 'float'
		self.units = None

	@property
	def wait(self):
		return str(self._wait)

	@wait.setter
	def wait(self, value):
		wait = Quantity(value)
		wait.assert_dimensions('s')

		self._wait = wait

	def with_type(self, value):
		"""
		Set to the correct type, and wrap with the correct units.
		"""

		if self.type == 'integer':
			return int(value)
		elif self.type == 'float':
			return value
		elif self.type == 'quantity' and self.units is not None:
			return Quantity(value, self.units)
		else:
			raise ValueError('Invalid variable setup; type: {0}, units: {1}'.format(self.type, self.units))

	def __iter__(self):
		if self.use_const:
			iterable = [self.const]
		else:
			iterable = self.config

		return (self.with_type(x) for x in iterable)

	def __len__(self):
		if self.use_const:
			return 1
		else:
			return len(self.config)

	def __str__(self):
		found_values = list(islice(iter(self), 0, self.search_values + 1))

		if isinstance(found_values[0], Quantity):
			found_values = [x.original_value for x in found_values]

		shown_values = ', '.join('{0:g}'.format(x) for x in found_values[:self.display_values])

		if len(found_values) > self.display_values:
			if len(found_values) > self.display_values + 1:
				shown_values += ', ...'

			if len(found_values) <= self.search_values:
				shown_values += ', {0:g}'.format(found_values[-1])

		smooth_from = '(' if not self.use_const and self.smooth_from else '['
		smooth_to = ')' if not self.use_const and self.smooth_to else ']'
		units = ' {0}'.format(self.units) if self.units is not None else ''
		return '{0}{1}{2}{3}'.format(smooth_from, shown_values, smooth_to, units)


class LinSpaceConfig(object):
	"""
	Linear space variable configuration.
	"""

	def __init__(self, initial=0.0, final=0.0, steps=1):
		self.initial = initial
		self.final = final
		self.steps = steps

	@property
	def steps(self):
		return self._steps

	@steps.setter
	def steps(self, value):
		if value <= 0:
			raise ValueError('Number of steps must be positive, not "{0}".'.format(value))

		self._steps = value

	def __iter__(self):
		return iter(numpy.linspace(self.initial, self.final, self.steps))

	def __len__(self):
		return self.steps


class ArbitraryConfig(object):
	"""
	Variable configuration for arbitrary values.
	"""

	def __init__(self, values):
		self.values = values

	def __iter__(self):
		return iter(self.values)

	def __len__(self):
		return len(self.values)



from ...mock.mock_abstract_device import MockAbstractDevice
from ..smf100a import SMF100A

"""
Mock SMF100A signal generator.
"""


class MockSMF100A(MockAbstractDevice, SMF100A):
	"""
	Mock interface for the R&S SMF100A.
	"""

	def __init__(self, *args, **kwargs):
		self.mocking = SMF100A

		MockAbstractDevice.__init__(self, *args, **kwargs)

	def _reset(self):
		self.mock_state['enabled'] = 0
		self.mock_state['power'] = 0.007 # V
		self.mock_state['frequency'] = 1e10 # Hz

	def write(self, message, result=None, done=False):
		if not done:
			cmd, args, query = self._split_message(message)

			if cmd[0] == 'output':
				if cmd[1] == 'state':
					if query:
						result = self.mock_state['enabled']
					else:
						self.mock_state['enabled'] = int(args)
					done = True
			elif cmd[0] == 'source':
				if cmd[1] == 'power' and cmd[2] == 'power':
					if query:
						result = self.mock_state['power']
					else:
						self.mock_state['power'] = float(args)
					done = True
				elif cmd[1] == 'frequency' and cmd[2] == 'cw':
					if query:
						result = self.mock_state['frequency']
					else:
						self.mock_state['frequency'] = float(args)
					done = True
			elif cmd[0] == 'unit':
				done = True

		MockAbstractDevice.write(self, message, result, done)


name = 'SMF100A'
implementation = MockSMF100A



global config

# Addresses of devices with which to test.
config['devices'] = {}
config['devices']['AWG5014B'] = {
	'address': {'ip_address': '192.0.2.123'},
	'manufacturer': 'Tektronix',
	'model': 'AWG5014B',
}
config['devices']['DM34410A'] = {
	'address': {'gpib_board': 0, 'gpib_pad': 1},
	'manufacturer': 'Agilent',
	'model': '34410A',
}
config['devices']['VoltageSource'] = {
	'address': {'usb_resource': 'USB::0x1234::0x5678::01234567::RAW'},
	'manufacturer': 'IQC',
	'model': 'Voltage source',
}



import logging
log = logging.getLogger(__name__)

from nose.tools import eq_
from unittest import main

from spacq.interface.units import Quantity
from spacq.tests.tool.box import DeviceServerTestCase

from ... import dpo7104


class DPO7104Test(DeviceServerTestCase):
	def obtain_device(self):
		return DeviceServerTestCase.obtain_device(self, impl=dpo7104.DPO7104,
				manufacturer='Tektronix', model='DPO7104')

	def testAcquire(self):
		"""
		Obtain some waveforms.
		"""

		dpo = self.obtain_device()
		dpo.reset()

		dpo.autoset()

		eq_(dpo.stopafter, 'runstop')
		assert dpo.acquiring

		ws = []

		# 1 is enabled by default.
		# 2 and 3 are disabled by default.
		dpo.channels[4].enabled = True

		dpo.channels[1].scale = dpo.channels[4].scale = Quantity(500, 'mV')
		dpo.channels[1].offset = dpo.channels[4].offset = Quantity(1, 'V')

		# Many records.
		dpo.time_scale = Quantity(100, 'ns')
		dpo.sample_rate = Quantity(40, 'GHz')
		eq_(dpo.record_length, 4e3)

		dpo.acquire()
		ws.append(dpo.channels[1].waveform)
		ws.append(dpo.channels[4].waveform)

		eq_(dpo.time_scale.value, 1e-7)
		eq_(dpo.sample_rate.value, 4e10)

		eq_(len(ws[0]), 4e3)
		eq_(len(ws[1]), 4e3)

		# Long sample.
		dpo.acquisition_mode = 'sample'

		dpo.time_scale = Quantity(10, 's')
		dpo.sample_rate = Quantity(0.1, 'kHz')
		eq_(dpo.record_length, 1e3)

		dpo.acquire()
		ws.append(dpo.channels[1].waveform)
		ws.append(dpo.channels[4].waveform)

		eq_(dpo.time_scale.value, 1e1)
		eq_(dpo.sample_rate.value, 1e2)
		eq_(len(ws[2]), 1e3)
		eq_(len(ws[3]), 1e3)

		# Check the channels.
		assert     dpo.channels[1].enabled
		assert not dpo.channels[2].enabled
		assert not dpo.channels[3].enabled
		assert     dpo.channels[4].enabled
		# Check the data.
		assert all(x >= -1.5 and x <= 3.5 for w in ws for _, x in w)


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

from ..abstract_device import AbstractDevice

"""
Mock hardware device.
"""


class MockAbstractDevice(AbstractDevice):
	"""
	A class for controlling fake devices.
	"""

	output = None

	@staticmethod
	def _split_message(message):
		"""
		Split a message into usable components.
		"""

		message = message.split(None, 1)
		try:
			cmd, args = message[0], message[1].strip()
		except IndexError:
			cmd, args = message[0], ''

		if cmd[-1] == '?':
			query = True
			cmd = cmd[:-1]
		else:
			query = False
		cmd = cmd.split(':')

		return cmd, args, query

	def __init__(self, autoconnect=True, *args, **kwargs):
		"""
		Create a device, always successfully.
		"""

		log.info('Creating mock device.')

		self.multi_command_responses = None
		self.mock_state = {}

		try:
			self._reset()
			self.mocking._setup(self)
		except AttributeError:
			# An instance of MockAbstractDevice itself.
			AbstractDevice._setup(self)

		if autoconnect:
			self.connect()

		log.info('Created mock device "{0}".'.format(self.name))

	def _reset(self):
		"""
		Reset to a known blank state.
		"""

		# Each mock device has its own mock_state to worry about.
		pass

	def connect(self):
		"""
		Pretend to connect.
		"""

		self._connected()

	def multi_command_start(self):
		"""
		Start buffering responses.
		"""

		self.multi_command_responses = []

	def multi_command_stop(self):
		"""
		Return buffered responses.
		"""

		responses = self.multi_command_responses
		self.multi_command_responses = None

		return responses

	def write(self, message, result=None, done=False):
		"""
		Act on what is being written.
		"""

		log.debug('Writing to device: {0!r}'.format(message))

		if not done:
			if message == '*idn?':
				result = self.name
				done = True
			elif message == '*opc?':
				result = 1
				done = True
			elif message in ['*rst', 'system:preset']:
				self._reset()
				done = True
			elif message == 'system:version?':
				result = '42'
				done = True

		if not done:
			raise NotImplementedError('Cannot understand message: {0!r}'.format(message))

		if result is None:
			self.output = None
		else:
			self.output = str(result) + '\n'

	def read_raw(self, **kwargs):
		"""
		Return the result of the last write operation.
		"""

		log.debug('Read from device: {0!r}'.format(self.output))

		return self.output

	def ask(self, *args, **kwargs):
		"""
		Ask, but possibly buffer the answer.
		"""

		result = AbstractDevice.ask(self, *args, **kwargs)

		if self.multi_command_responses is None:
			return result
		else:
			self.multi_command_responses.append(result)

	def close(self):
		"""
		Pretend to close the connection.
		"""

		log.debug('Closing device: {0}'.format(self.name))

	@property
	def idn(self):
		self.ask('*idn?')

	@property
	def opc(self):
		self.ask('*opc?')



import random

from ...mock.mock_abstract_device import MockAbstractDevice
from ..abc1234 import ABC1234

"""
Mock Sample ABC1234
"""


class MockABC1234(MockAbstractDevice, ABC1234):
	"""
	Mock interface for the Sample ABC1234.
	"""

	def __init__(self, *args, **kwargs):
		self.mocking = ABC1234

		MockAbstractDevice.__init__(self, *args, **kwargs)

	def _reset(self):
		self.mock_state['setting'] = 'default value'

	def write(self, message, result=None, done=False):
		if not done:
			cmd, args, query = self._split_message(message)

			if cmd[0] == 'some':
				if cmd[1] == 'setting':
					if query:
						result = self.mock_state['setting']
					else:
						self.mock_state['setting'] = args
					done = True
			elif cmd[0] == 'read' and query:
				result = '-1.{0:04d}0000E-02'.format(random.randint(0, 9999))
				done = True

		MockAbstractDevice.write(self, message, result, done)


name = 'ABC1234'
implementation = MockABC1234



#!/usr/bin/env python2

import logging
logging.basicConfig(level=logging.WARNING)

import wx

from spacq import VERSION
from spacq.gui.action.data_capture import DataCapturePanel
from spacq.gui.action.smooth_reset import SmoothResetPanel
from spacq.gui.config.devices import DeviceConfigFrame
from spacq.gui.config.pulse import PulseProgramFrame
from spacq.gui.config.variables import VariablesPanel
from spacq.gui.display.plot.live.list import ListMeasurementFrame
from spacq.gui.display.plot.live.scalar import ScalarMeasurementFrame
from spacq.gui.global_store import GlobalStore
from spacq.gui.tool.box import MessageDialog


class SweepingAcquisitionFrame(wx.Frame):
	def __init__(self, parent, global_store, *args, **kwargs):
		wx.Frame.__init__(self, parent, *args, **kwargs)

		# Frame.
		frame_box = wx.BoxSizer(wx.VERTICAL)

		## Variables.
		self.variables_panel = VariablesPanel(self, global_store)
		self.variables_panel.SetMinSize((800, 300))
		frame_box.Add(self.variables_panel, proportion=1, flag=wx.EXPAND)

		## Bottom.
		bottom_box = wx.BoxSizer(wx.HORIZONTAL)
		frame_box.Add(bottom_box, flag=wx.EXPAND)

		### Data capture.
		self.data_capture_panel = DataCapturePanel(self, global_store, style=wx.BORDER_RAISED)
		bottom_box.Add(self.data_capture_panel, proportion=1, flag=wx.EXPAND)

		### Smooth reset.
		self.smooth_reset_panel = SmoothResetPanel(self, global_store, style=wx.BORDER_RAISED)
		bottom_box.Add(self.smooth_reset_panel, flag=wx.EXPAND)

		self.SetSizerAndFit(frame_box)

		self.Bind(wx.EVT_CLOSE, self.OnClose)

	def OnClose(self, evt):
		if self.data_capture_panel.capture_dialogs > 0:
			msg = 'Cannot exit, as a sweep is currently in progress.'
			MessageDialog(self, msg, 'Sweep in progress').Show()

			evt.Veto()
		else:
			evt.Skip()


class AcquisitionApp(wx.App):
	def OnInit(self):
		self.global_store = GlobalStore()

		# Frames.
		self.acq_frame = SweepingAcquisitionFrame(None, self.global_store, title='Acquisition')
		self.device_config_frame = None
		self.pulse_program_frame = None

		# Menu.
		menuBar = wx.MenuBar()

		## Configuration.
		menu = wx.Menu()
		menuBar.Append(menu, '&Configuration')

		### Devices.
		item = menu.Append(wx.ID_ANY, '&Devices...')
		self.Bind(wx.EVT_MENU, self.OnMenuConfigurationDevices, item)

		### Measurements.
		submenu = wx.Menu()
		menu.AppendMenu(wx.ID_ANY, '&Measurements', submenu)

		item = submenu.Append(wx.ID_ANY, 'Add &scalar...')
		self.Bind(wx.EVT_MENU, self.OnMenuConfigurationMeasurementsAddScalar, item)

		item = submenu.Append(wx.ID_ANY, 'Add &list...')
		self.Bind(wx.EVT_MENU, self.OnMenuConfigurationMeasurementsAddList, item)

		### Pulse program.
		item = menu.Append(wx.ID_ANY, '&Pulse program...')
		self.Bind(wx.EVT_MENU, self.OnMenuConfigurationPulseProgram, item)

		## Help.
		menu = wx.Menu()
		menuBar.Append(menu, '&Help')

		### About.
		item = menu.Append(wx.ID_ABOUT, '&About...')
		self.Bind(wx.EVT_MENU, self.OnMenuHelpAbout, item)

		self.acq_frame.SetMenuBar(menuBar)

		# Display.
		self.acq_frame.SetSizerAndFit(self.acq_frame.Sizer)
		self.acq_frame.Show()
		self.SetTopWindow(self.acq_frame)
		self.acq_frame.Raise()

		return True

	def OnMenuConfigurationDevices(self, evt=None):
		def close_callback():
			self.device_config_frame = None

		if self.device_config_frame is None:
			self.device_config_frame = DeviceConfigFrame(self.acq_frame, self.global_store, close_callback)
			self.device_config_frame.Fit()
			self.device_config_frame.Show()

		self.device_config_frame.Raise()

	def OnMenuConfigurationMeasurementsAddScalar(self, evt=None):
		measurement_frame = ScalarMeasurementFrame(self.acq_frame, self.global_store)
		measurement_frame.Show()

	def OnMenuConfigurationMeasurementsAddList(self, evt=None):
		measurement_frame = ListMeasurementFrame(self.acq_frame, self.global_store)
		measurement_frame.Show()

	def OnMenuConfigurationPulseProgram(self, evt=None):
		def close_callback():
			self.pulse_program_frame = None

		if self.pulse_program_frame is None:
			self.pulse_program_frame = PulseProgramFrame(self.acq_frame, self.global_store, close_callback)
			self.pulse_program_frame.Fit()
			self.pulse_program_frame.Show()

		self.pulse_program_frame.Raise()

	def OnMenuHelpAbout(self, evt=None):
		info = wx.AboutDialogInfo()
		info.SetName('Acquisition')
		info.SetDescription(
			'An application for sweeping device values and acquiring data.\n'
			'\n'
			'Using Spanish Acquisition version {0}.'.format(VERSION)
		)

		wx.AboutBox(info)


if __name__ == "__main__":
	app = AcquisitionApp()
	app.MainLoop()



from nose.tools import eq_
from numpy import arange, linspace, repeat
from numpy.testing import assert_array_equal, assert_array_almost_equal
from pubsub import pub
from threading import RLock, Thread
import time
from unittest import main, TestCase

from .. import box


class FlattenTest(TestCase):
	def testEmpty(self):
		"""
		Flatten nothing.
		"""

		eq_(list(box.flatten([])), [])

	def testSingle(self):
		"""
		Flatten one thing.
		"""

		eq_(list(box.flatten([(1, 2, 3, 4, 5, 6, 7)])), [1, 2, 3, 4, 5, 6, 7])

	def testMany(self):
		"""
		Flatten all the things.
		"""

		eq_(list(box.flatten([(1, 2, 3), [4, 5, 6], {7: 8}])), [1, 2, 3, 4, 5, 6, 7])


class SiftTest(TestCase):
	def testEmpty(self):
		"""
		Sift nothing.
		"""

		eq_(box.sift([], object), [])

	def testAllSame(self):
		"""
		Either keep all or remove all.
		"""

		items = [object() for _ in xrange(5)]

		eq_(box.sift(items, object), items)
		eq_(box.sift(items, Exception), [])

	def testVaried(self):
		"""
		All sorts of objects.
		"""

		items = [ValueError(), TypeError(), KeyError(), 5]

		eq_(box.sift(items, object), items)
		eq_(box.sift(items, int), [items[3]])
		eq_(box.sift(items, Exception), items[0:3])
		eq_(box.sift(items, ValueError), [items[0]])


class TriplesToMeshTest(TestCase):
	def testSimple(self):
		"""
		No interpolation required.
		"""

		x = [1, 2, 3, 4] * 3 # [1, 2, 3, 4, 1, ...]
		y = repeat([5, 6, 7], 4) # [5, 5, 5, 5, 6, ...]
		z = linspace(0, -11, 12) # [0, -1, -2, -3, ...]

		result, x_bounds, y_bounds, z_bounds = box.triples_to_mesh(x, y, z)

		assert_array_equal(result, z.reshape(3, 4))
		eq_(x_bounds, (1, 4))
		eq_(y_bounds, (5, 7))
		eq_(z_bounds, (-11, 0))

	def testInterpolated(self):
		"""
		Some interpolation required.
		"""

		x = [0, 0, 0.25, 1, 1]
		y = [0, 1,    0, 0, 1]
		z = [1, 2,  1.5, 3, 4]

		result, x_bounds, y_bounds, z_bounds = box.triples_to_mesh(x, y, z)

		expected = [
			[1, 2, 3],
			[2, 3, 4],
		]

		assert_array_almost_equal(result, expected)
		eq_(x_bounds, (0, 1))
		eq_(y_bounds, (0, 1))
		eq_(z_bounds, (1, 4))

	def testBigDataSet(self):
		"""
		One hundred thousand evenly-spaced data points.
		"""

		x = range(1000) * 100
		y = repeat(range(100), 1000)
		z = arange(0, 100000)

		result, x_bounds, y_bounds, z_bounds = box.triples_to_mesh(x, y, z)

		assert_array_equal(result, z.reshape(100, 1000))
		eq_(x_bounds, (0, 999))
		eq_(y_bounds, (0, 99))
		eq_(z_bounds, (0, 99999))


class EnumTest(TestCase):
	def testEmpty(self):
		"""
		A useless object.
		"""

		e = box.Enum()

		try:
			e.anything
		except AttributeError:
			pass
		else:
			assert False, 'Expected AttributeError.'

		eq_(len(e), 0)

	def testNotEmpty(self):
		"""
		A regular enum.
		"""

		values = ['cow', 'moon', 'dish', 'spoon']

		e = box.Enum(values)

		eq_(len(e), len(values))

		for v in values:
			eq_(getattr(e, v), v)

		e.cow, e.moon, e.dish, e.spoon

		try:
			e.diddle
		except AttributeError:
			pass
		else:
			assert False, 'Expected AttributeError.'

	def testDuplicates(self):
		"""
		Check uniqueness and equality testing.
		"""

		e = box.Enum(['a'] * 100 + ['b'] * 50)
		f = box.Enum(['b', 'a'])

		eq_(e, f)


class PubDictTest(TestCase):
	def testSimple(self):
		"""
		Run through some valid cases.
		"""

		data = []

		def msg_added(name, value):
			data.append((name, value))

		def msg_removed(name):
			data.append((name,))

		p = pub.Publisher()

		p.subscribe(msg_added, 'test.added')
		p.subscribe(msg_removed, 'test.removed')

		pd = box.PubDict(RLock(), p, 'test')
		pd['a'] = 'abc'
		pd['b'] = 'def'
		pd['c'] = 'ghi'
		del pd['a']
		pd['a'] = 'jkl'
		del pd['c']

		expected = {'a': 'jkl', 'b': 'def'}
		expected_data = [('a', 'abc'), ('b', 'def'), ('c', 'ghi'), ('a',), ('a', 'jkl'), ('c',)]

		eq_(pd, expected)
		eq_(data, expected_data)

	def testLocked(self):
		"""
		Attempt a compound operation.
		"""

		pd = box.PubDict(RLock(), pub.Publisher(), 'test')

		pd['a'] = 'abc'

		with pd.lock:
			del pd['a']
			pd['a'] = 'def'

		expected = {'a': 'def'}

		eq_(pd, expected)

	def testInvalid(self):
		"""
		Try some bad scenarios.
		"""

		pd = box.PubDict(RLock(), pub.Publisher(), 'test')

		# Setting None.
		try:
			pd['a'] = None
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'

		# Overwriting.
		pd['a'] = 'abc'
		try:
			pd['a'] = 'def'
		except KeyError:
			pass
		else:
			assert False, 'Expected KeyError.'


class SynchronizedTest(TestCase):
	class SynchronizedObject(object):
		def __init__(self):
			self.buf = []

			self.lock = RLock()

		@box.Synchronized()
		def do(self, values):
			for i in xrange(values):
				self.buf.append(i)
				time.sleep(0.001)


	class SynchronizedThread(Thread):
		def __init__(self, obj, times, values):
			Thread.__init__(self)

			self.obj = obj
			self.times = times
			self.values = values

		def run(self):
			for i in xrange(self.times):
				self.obj.do(self.values)


	def testSynchronization(self):
		"""
		Ensure that synchronized methods are called in the correct order.
		"""

		num_threads = 4

		times = 4
		values = 5

		obj = SynchronizedTest.SynchronizedObject()

		thrs = []
		for _ in xrange(num_threads):
			thrs.append(SynchronizedTest.SynchronizedThread(obj, times, values))

		for thr in thrs:
			thr.start()

		for thr in thrs:
			thr.join()

		eq_(obj.buf, range(values) * times * num_threads)


class WithoutTest(TestCase):
	def testWith(self):
		"""
		Use Without in with.
		"""

		with box.Without():
			with box.Without() as w:
				assert w is None

	def testException(self):
		"""
		Ensure that exceptions are passed through.
		"""

		try:
			with box.Without():
				raise IndexError
		except IndexError:
			pass
		else:
			assert False, 'Expected IndexError.'


if __name__ == '__main__':
	main()
